<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: settings.php 440 2010-08-20 23:01:05Z stian $
 * @category	Ninja
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

class ComNinjaModelSettings extends KModelTable
{
	public function __construct(KConfig $options)
	{
		parent::__construct($options);
		
		//lets get our states
	   	$this->_state
	   		 ->insert('active', 'boolean', false)
	   		 ->insert('default', 'boolean', false);
	}
	
	/**
	 * Method to get a KConfig object which represents a table row params
	 *
	 * If no id state are set, and the application isn't JAdmin, 
	 * then the current page parameters will be checked for an id.
	 * If there are no id state, or page parameter settings id, the default setting will be fetched
	 *
	 * @return KConfig
	 */
	public function getParams()
	{
		if(!$this->_state->id && !KFactory::get('lib.koowa.application')->isAdmin())
		{
			$params = $this->_getPageParameters();
			$package = $this->getIdentifier()->package;
			if(array_key_exists($package.'_setting_id', $params)) $this->_state->id = $params[$package.'_setting_id'];
			//die('<pre>'.print_r($params, true).'</pre>');
		}
		//die('<pre>'.print_r(KRequest::get('session', 'raw'), true).'</pre>');
	
		if (!isset($this->_params))
		{
			$table  = KFactory::get($this->getTable());
   			$query = $table->getDatabase()->getQuery();

			$this->_buildQueryColumns($query);
			$this->_buildQueryFrom($query);
			$this->_buildQueryJoins($query);
			$this->_buildQueryWhere($query);
			$this->_buildQueryGroup($query);
			$this->_buildQueryHaving($query);
			
			$query->where('default', '=', 1, 'or')->order('default', 'asc');

		 	$this->_params = $table->select($query, KDatabase::FETCH_ROW);

		 	if(!KFactory::get('lib.koowa.application')->isAdmin())
		 	{
		 		$params = $this->_getPageParameters();
		 		$params = new KConfig($params);
		 		//if(!is_object($this->_params->params)) die(__CLASS__.'<pre>'.var_export($this->_params->params, true).'</pre>');
		 		$this->_params->params = $params->append($this->_params->params->toArray());
		 	}
		}

		return $this->_params->params;
	}
	
	/**
	 * KFactory::get('lib.koowa.application')->getPageParameters() doesn't always work as expected, so we do this
	 *
	 * @author Stian Didriksen <stian@ninjaforge.com>
	 * @return array 	page parameter values
	 */
	protected function _getPageParameters()
	{
		$menus	= JSite::getMenu();
		$menu	= $menus->getActive() ? $menus->getActive() : $menus->getDefault();
		$params	= new JParameter($menu->params);
		return (array) $params->_registry['_default']['data'];
	}
		
	public function getItem()
	{
		if(!$this->_item)
		{
			$identifier = $this->getIdentifier();
			$isNotSetting = KRequest::get('get.view', 'cmd') != 'setting';
			
			$params = array();
			if(!KFactory::get('lib.koowa.application')->isAdmin())
			{
				$menus	= JSite::getMenu();
				$menu	= $menus->getActive() ? $menus->getActive() : $menus->getDefault();
				$settings	= new JParameter($menu->params);
				$params = (array) $settings->_registry['_default']['data'];
				$pk		= KFactory::get($this->getTable())->getPrimaryKey();
				$name	= $pk['id']->name;

				$id = !empty($params[$name]) ? $params[$name] : false;
				if($id)
				{
					$table = KFactory::get($this->getTable());
					
					$item = $table->select($id, KDatabase::FETCH_ROW);
				}
				else if($isNotSetting)
				{
					$item = $this->getDefault();
				}
			} else {
				if($isNotSetting)
				{
					$item = $this->getDefault();
				}
				else
				{
					$table = KFactory::get($this->getTable());
					
					$item = $table->select($this->_state->id, KDatabase::FETCH_ROW);
				}
			}
	
	
			$params			= new KConfig($params);
			$item->params	= $params->append($item->params->toArray());
			$item->xml		= simplexml_load_file($table->getXMLPath());
			
			$this->_item = $item;
		}

		return $this->_item;
	}
	
	public function getDefault()
	{
		$table = KFactory::get($this->getTable());
		$query = $table->getDatabase()->getQuery()
					->where('default', '=', 1);
		
		return $table->select($query, KDatabase::FETCH_ROW);
	}
	
	/**
	 * Query WHERE clause
	 */
	protected function _buildQueryWhere(KDatabaseQuery $query)
	{
		parent::_buildQueryWhere($query);
	
		if($search = $this->_state->search)
		{
			$query->where('tbl.title', 'LIKE', '%'.$search.'%');
		}
	}
}