<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: dashboard.php 778 2010-08-28 18:10:55Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard Default Controller
 *
 * @package Ninjaboard
 */
class ComNinjaboardControllerDashboard extends ComNinjaControllerDashboard
{
	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		parent::__construct($config);

		if( !isset($this->_request->tmpl) || ( isset($this->_request->tmpl) && $this->_request->tmpl != 'component' ) )
		{
			$this->registerFunctionBefore('read', array('checkMigration', 'checkInstall'));
		}
	}
	
	/**
	 * Checks if the site have migrated from 0.5, and got no data yet.
	 * If true, then it'll display a message pointing to the Tools screen and the NB importer.
	 *
	 * @author Stian Didriksen <stian@ninjaforge.com>
	 */
	public function checkMigration()
	{
		try {
			$migrated = KFactory::get('admin::com.ninjaboard.model.forums_backups')->getTotal();
		} catch(KDatabaseTableException $e) {
			// Do nothing if the table don't exist
			return $this;
		}
		
		//Do nothing if there's no migrated data
		if(!$migrated) return $this;

		//Do nothing if there's already data in Ninjaboard
		$existing = KFactory::get('admin::com.ninjaboard.model.forums')->getTotal();
		if($existing) return $this;
		
		JError::raiseNotice(0, sprintf(
			JText::_('Congratulations on the upgrade! You\'ll find that all your 0.5 data were backed up during the 1.0 install. %s'),
			'<a href="'.JRoute::_('&option=com_ninjaboard&view=tools&shortcut=legacy').'">'.
			JText::_('Click here to import.').
			'</a>'
		));
		KFactory::get('lib.koowa.application')->enqueueMessage(sprintf(
			JText::_('%s You can downgrade to 0.5 anytime with your data restored. Uninstall 1.0, 0.5 data are restored automatically. Then simply reinstall 0.5.'),
			'<em>'.
			JText::_('Did you know?').
			'</em>'	
		));
	}

	/**
	 * Checks if the site is a clean and fresh install
	 * If true, then it'll display a message pointing to the Tools screen and the Sample Content importer.
	 *
	 * @author Stian Didriksen <stian@ninjaforge.com>
	 */
	public function checkInstall()
	{
		//Do nothing if there's already data in Ninjaboard
		$existing = KFactory::get('admin::com.ninjaboard.model.forums')->getTotal();
		if($existing) return $this;
	
		// Do nothing if the migration table exist
		try {
			$migrated = KFactory::get('admin::com.ninjaboard.model.forums_backups')->getTotal();
		} catch(KDatabaseTableException $e) {
			JError::raiseNotice(0, sprintf(
				JText::_('Thank you for choosing Ninjaboard! Get started faster by importing some sample content. %s'),
				'<a href="'.JRoute::_('&option=com_ninjaboard&view=tools&shortcut=demo').'">'.
				JText::_('Click here to import.').
				'</a>'
			));
		}
	}

	/**
	 * Display the view
	 *
	 * This is temporarily needed while we run the JCE badge
	 *
	 * @param  KCommandContext $context
	 */
	public function displayView(KCommandContext $context)
	{
		$img  = KFactory::get('admin::com.ninja.helper')->img('/jce_logo_black.png');
		$text = json_encode(sprintf(JText::_('Sponsored by %s'), '<img src="'.$img.'" alt="JCE logo" />'));
	
		KFactory::get('lib.joomla.document')->addScriptDeclaration("
			window.addEvent('domready', function(){
				if(td = document.getElements('table.nf-extension td')[1]) td.grab(new Element('a', {
					'class': 'sponsor',
					html: $text,
					href: 'http://www.joomlacontenteditor.net/',
					target: '_blank',
					styles: {
						display: 'block'
					}
				}));
			});
		");
		KFactory::get('lib.joomla.document')->addStyleDeclaration("
			.sponsor:hover {
				text-decoration: none;
			}
			.sponsor img {
				vertical-align: middle;
				width: 135px;
			}
		");
	
		return parent::displayView($context);
	}
}