<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: user.php 810 2010-09-01 13:57:16Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard User Controller
 *
 * @package Ninjaboard
 */
class ComNinjaboardControllerUser extends ComNinjaControllerView
{
	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $options)
	{
		parent::__construct($options);
		
		$this	
				->registerFunctionBefore('add',	'setUserData')
				->registerFunctionBefore('edit',	'setUserData')
				->registerFunctionAfter('upload',	'setAvatar');
	}
	
	public function setAvatar()
	{
		if(!$avatar = KRequest::get('files.uploaded', 'string', false)) return;
		
		$data = array('avatar' => $avatar);
		$data['id'] = $this->getRequest()->id;
		
		$rowset = KFactory::tmp('admin::com.ninjaboard.model.people')
				->set($this->getRequest())
				->getItem()
				->setData($data)
				->save();		
	}
	
	public function getUploadDestination()
	{
		$identifier = $this->getIdentifier();
		$option		= $identifier->type.'_'.$identifier->package;
		$user		= KFactory::get($this->getModel())->getItem();
		$path		= '/media/'.$option.'/images/avatars/'.KRequest::get('get.id', 'int', $user->id).'/';
		return $path;
	}
	
	/*
	 * Generic save action
	 *
	 *	@param	mixed 	Either a scalar, an associative array, an object
	 * 					or a KDatabaseRow
	 * @return KDatabaseRow 	A row object containing the saved data
	 */
	protected function _actionSave($data)
	{
		$row = $this->execute('edit', $data);
		
		$this->_redirect = KRequest::get('session.com.dispatcher.referrer', 'url');
		return $row;
	}

	/*
	 * Generic apply action
	 *
	 *	@param	mixed 	Either a scalar, an associative array, an object
	 * 					or a KDatabaseRow
	 * @return 	KDatabaseRow 	A row object containing the saved data
	 */
	protected function _actionApply($data)
	{
		$row = $this->execute('edit', $data);

		$this->_redirect = 'view='.$this->_identifier->name.'&id='.$row->id;
		return $row;
	}

	/**
	 * Generic edit action, saves over an existing item
	 *
	 * @param	mixed 	Either a scalar, an associative array, an object
	 * 					or a KDatabaseRow
	 * @return KDatabaseRowset 	A rowset object containing the updated rows
	 */
	protected function _actionEdit($data)
	{
		$data['id'] = $this->getRequest()->id;
	
		$rowset = KFactory::tmp('admin::com.ninjaboard.model.people')
				->set($this->getRequest())
				->getItem()
				->setData($data);

		$rowset->save();

		return $rowset;
	}

	public function setUserData(KCommandContext $context)
	{
		//if(KRequest::type() == 'FLASH') return true;
		$data = $context->data;
		$id   = $this->getRequest()->id;

		if(isset($data->usergroup))
		{
			$data = array('ninjaboard_user_group_id' => $data['usergroup']);
			KFactory::tmp('admin::com.ninjaboard.model.usergroup_maps')
				->id($id)
				->getList()
				->delete();

			if($data['ninjaboard_user_group_id'][0] === '0') return $this;

			foreach($data['ninjaboard_user_group_id'] as $group)
			{
				
				$usergroup = KFactory::tmp('admin::com.ninjaboard.model.usergroup_maps')
					->getItem()
					->setData(array(
						'id' => $id,
						'ninjaboard_user_group_id' => $group
					))
					->save();					
			}
		}
	}

	/**
	 * Generic method to set model states
	 *
	 * @return KDatabaseRow 	A row object containing the filtered data
	 */
	protected function _setModelState()
	{
		parent::_setModelState();

		if($group = KRequest::get('get.f.usergroup', 'int')) $this->getModel()->setState('usergroup', $group);
		return $this;
	}
	
	protected function _actionApplys($data)
	{
		$result = parent::_actionApply($data);
		
		$this->_redirect = 'view=user&id='.KRequest::get('get.id', 'int');
		
		return $result;
	}
	
//	public function getRedirect()
//	{
//		if(KRequest::type() == 'FLASH') return array();
//		return parent::getRedirect();
//	}
}