<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: abstract.php 754 2010-08-19 21:42:45Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * ComNinjaboardConverterAbstract
 *
 * Just to keep things DRY.
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
abstract class ComNinjaboardConverterAbstract extends KObject implements ComNinjaboardConverterInterface, KObjectIdentifiable
{
	/**
	 * The current data
	 *
	 * @var array
	 */
	public $data = array();
	
	/**
	 * If set to true, then this converter is able to run in steps
	 *
	 * @var bool
	 */
	public $splittable = false;
	
	/**
	 * Array over tructated tables to prevent tables from trunctate twice
	 *
	 * @var array
	 */
	public $trunctated = array();

	/**
	 * Get the object identifier
	 * 
	 * @return	KIdentifier	
	 * @see 	KObjectIdentifiable
	 */
	public function getIdentifier()
	{
		return $this->_identifier;
	}

	/**
	 * Execute the convertion
	 *
	 * @param  $dbprefix string	Used by converters like phpBB3 to allow importing from multiple tables,
	 * 							 and by multiple prefixes to avoid the duplicates blocker to block'em
	 * @return $this
	 */
	public function convert($dbprefix = false)
	{
		if(!$dbprefix) $dbprefix = KFactory::get('lib.joomla.config')->getValue('dbprefix');
		$identifier = new KIdentifier('admin::com.ninjaboard.table.default');

		foreach($this->data as $this->name => $rows)
		{
			$identifier->name = $this->name;
			$table = KFactory::get($identifier);
			
			if(KRequest::get('post.offset', 'int', 0) < 1) $this->_truncateTable($table);
			
			foreach($rows as $row)
			{	
				/*	
				$this->row = KFactory::tmp($identifier)
							->getItem()
							->setData($this->row)
							->save(); 
				//*/

				//Filter the data and remove unwanted columns
				$data = $table->filter($row, true);
				
				//Get the data and apply the column mappings
				$data = $table->mapColumns($data);
				
				//@TODO find a better way to encode KConfig to json to avoid fatal errors
				foreach($data as $i => $val)
				{
					if(is_array($val)) $data[$i] = json_encode($val);
				}
				
				$table->getDatabase()->insert($table->getBase(), $data);
			}
		}

		return $this;
	}
	
	/**
	 * Truncates tables that data is being imported to
	 *
	 * @author Stian Didriksen <stian@ninjaforge.com>
	 * @param  KDatabaseTableAbstract $table	The table being deleted
	 * @return $this
	 */
	protected function _truncateTable(KDatabaseTableAbstract $table)
	{
		$name = $table->getBase();

		//If this table have been tructated ebfore, don't trunctate it again
		if(isset($this->trunctated[$name])) return $this;

		$sql = 'TRUNCATE TABLE '.$table->getDatabase()->quoteName('#__'.$name);

		//Execute the query
		$table->getDatabase()->execute($sql);
		
		//Update the trunctated array
		$this->trunctated[$name] = true;

		return $this;
	}
	
	/**
	 * Imports data
	 *
	 * Used by splittable importers
	 *
	 * @param  array $tables		Array over tables to import from
	 * @param  string $package		The identifier package
	 * @return ComNinjaboardConverterInterface $this
	 */
	public function importData(array $tables, $package)
	{
		$offset = KRequest::get('post.offset', 'int', false);
		foreach ( $tables as $table )
		{
			$name = $table['name'];
			$query = clone $table['query'];

			if($offset === false)
			{
				if(!isset($this->data[$name]) || $this->data[$name] == array()) {
					//$this->data[$name] = (int)KFactory::tmp('admin::com.'.$package.'.table.'.$name, $table['options'])->count(clone $query);
					$this->data[$name] = (int)KFactory::get('admin::com.default.table.'.$name.$table['options']['base'], $table['options'])->count(clone $query);
				} else {
					//$this->data[$name] += (int)KFactory::tmp('admin::com.'.$package.'.table.'.$name, $table['options'])->count(clone $query);
					$this->data[$name] += (int)KFactory::get('admin::com.default.table.'.$name.$table['options']['base'], $table['options'])->count(clone $query);
				}

				continue;
			}
			elseif ($offset !== false)
			{
				$query->limit(KRequest::get('post.limit', 'int', 1000), $offset);
			}
			
			if(!isset($this->data[$name]) || $this->data[$name] == array()) {
				$this->data[$name] = KFactory::get('admin::com.default.table.'.$name.$table['options']['base'], $table['options'])
										->select($query, KDatabase::FETCH_ROWSET)
										->getData();
			} else {
				$rows = KFactory::get('admin::com.default.table.'.$name.$table['options']['base'], $table['options'])
							->select($query, KDatabase::FETCH_ROWSET)
							->getData();

				foreach($rows as $row)
				{
					$this->data[$name][] = $row;
				}
			}
			
		}

		if($offset === false)
		{
			$total = array_reduce($this->data, 'max');
			$steps = floor($total / KRequest::get('post.limit', 'int', 1000));
			if($steps > 0) 
			{
				echo json_encode(array('splittable' => true, 'total' => $total, 'steps' => $steps));
				return false;
			}
			else
			{
				echo json_encode(array('splittable' => false));
				foreach ( $tables as $table )
				{
					$name = $table['name'];
					$query = clone $table['query'];

					if($this->data[$name] == array() || is_numeric($this->data[$name])) {
						$this->data[$name] = KFactory::get('admin::com.default.table.'.$name.$table['options']['base'], $table['options'])
												->select($query, KDatabase::FETCH_ROWSET)
												->getData();
					} else {
						$rows = KFactory::get('admin::com.default.table.'.$name.$table['options']['base'], $table['options'])
									->select($query, KDatabase::FETCH_ROWSET)
									->getData();
		
						foreach($rows as $row)
						{
							$this->data[$name][] = $row;
						}
					}
				}
			}
		}
	}
	
	public function mapClean()
	{
		$table = KFactory::get('admin::com.ninjaboard.table.imported_rows');
		$where = $table->getDatabase()->getQuery()
					->where('ninjaboard', '=', 0);

		$table->getDatabase()->delete($table->getBase(), $where);
	}
	
	/**
	 * Set old key for an row
	 *
	 * @return $this
	 */
	public function mapLeft()
	{
		if(!isset($this->row['id'])) return $this;
		$this->id = $this->row['id'];
		$this->row['backup_id'] = $this->row['id'];
		$this->_key_maps[$this->name][$this->id] = true;
		unset($this->row['id']);

		return $this;
	}
	
	/**
	 * Set old key for an row
	 *
	 * @return $this
	 */
	public function mapRight()
	{
		$this->_key_maps[$this->name][$this->id] = $this->row->id;

		return $this;
	}
	
	/**
	 * Set old key for an row
	 *
	 * @return $this
	 */
	public function map($map, $key)
	{
		//$this->keys   = array_keys($this->_key_maps[$this->name]);
		//$this->values =	$this->_key_maps[$this->name];
		//$this->id = array_search($this->row->id, $this->_key_maps[$this->name]);

		if(!isset($this->_key_maps[$map])) return false;
		
		if(!isset($this->_key_maps[$map][$key])) return false;

		return $this->_key_maps[$map][$key];
	}
	
	protected function updateTopicForumIds()
	{
		foreach($this->data['topics'] as $topic)
		{
			KFactory::tmp('site::com.ninjaboard.model.topics')
				->id($topic->id)
				->getItem()
				->setData(array('forum_id' => $this->map('forums', $topic->forum_id)))
				->save();
		}
	}
	
	protected function updatePostTopicIds()
	{
		foreach($this->data['posts'] as $post)
		{
			$postid = $post->backup_id;
			$topic = $this->map('topics', $post->ninjaboard_topic_id);
			$row = KFactory::tmp('site::com.ninjaboard.model.posts')
					->getItem()
					->setData(
						array_merge(
							$post->getData(),
							array(
								'ninjaboard_topic_id' => $topic,
								'forum_id'			  => KFactory::tmp('site::com.ninjaboard.model.topics')
															->id($topic)
															->getItem()
															->forum_id
							)
						)
					)
					->save();
			$this->_key_maps['posts'][$postid] = $row->id;
		}
	}
	
	protected function updatePostIds()
	{	
		foreach($this->data['topics'] as $topic)
		{
			KFactory::tmp('site::com.ninjaboard.model.topics')
				->id($topic->id)
				->getItem()
				->setData(array('first_post_id' => $this->map('posts', $topic->first_post_id)))
				->save();
		}
	}
	
	protected function updateAttachments()
	{
		foreach($this->data['attachments'] as $attachment)
		{
			KFactory::tmp('site::com.ninjaboard.model.attachments')
				->id($attachment->id)
				->getItem()
				->setData(array('post_id' => $this->map('posts', $attachment->post_id)))
				->save();
		}
	}
}