<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: agora.php 754 2010-08-19 21:42:45Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * ComNinjaboardConverterKunena
 *
 * Imports data from Kunena.
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardConverterKunena extends ComNinjaboardConverterAbstract
{
	
	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $options)
	{
		$tables = array(
			'forums' => array(
				'options' => array(
					'name' => 'fb_categories',
					'base' => 'fb_categories',
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'tbl.*',
								'name AS title',
								'parent AS parent_id',
								'published AS enabled',
								'numTopics AS topics',
								'numPosts AS posts',
								'id_last_msg AS last_post_id'
							))
			),
			'topics' => array(
				'options' => array(
					'name' => 'fb_messages',
					'base' => 'fb_messages',
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'tbl.*',
								'catid AS forum_id',
								'topic_emoticon AS status',
								'id AS first_post_id',
								'moved AS moved_from_forum_id'
							))
							->where('parent', '=', 0)
			),
			'posts' => array(
				'options' => array(
					'name' => 'fb_messages',
					'base' => 'fb_messages',
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->set('columns', array('!`hold` AS `enabled`'))
							->select(array(
								'tbl.*',
								'thread AS ninjaboard_topic_id',
								'topic_emoticon AS status',
								'id AS first_post_id',
								'name AS guest_name',
								'email AS guest_email',
								'userid AS created_by',
								'time AS created_time',
								'ip AS user_ip',
								'modified_time AS mofidied_on',
								'modified_reason AS edit_reason',
								'message AS text',
							))
							->join('left', 'fb_messages_text', 'mesid = id')
			),
		);

		foreach ( $tables as $name => $table )
		{
			$options->data[$name] = KFactory::get('admin::com.kunena.table.'.$name, $table['options'])->select($table['query'])->getData();
		}
		
		parent::__construct($options);
	}
	
	/**
	 * Import the sample content
	 *
	 * @return $this
	 */
	public function convert()
	{
		parent::convert();

		$this->updateForumPaths();
		$this->updateTopicForumIds();
		$this->updatePostTopicIds();
		$this->updatePostIds();
		//$this->updateAttachments();

		return $this;
	}
	
	private function updateForumPaths()
	{
		// Update the parent id
		foreach($this['forums'] as $this->forum)
		{
			if($this->forum->parent_id == 0) continue;

			$row  = KFactory::tmp('admin::com.ninjaboard.model.forums')->id($this->forum->id)->getItem();
			$row->parent_id = $this->map('forums', $this->forum->parent_id);
			$row->save();
		}
		
		// Set the paths
		foreach($this['forums'] as $this->forum)
		{
			if($this->forum->parent_id > 0) continue;

			$this->updateForumPathsRecurse($this->forum);
		}
	}
	
	private function updateForumPathsRecurse($forum)
	{
		$query = KFactory::tmp('lib.koowa.database.query')->where('parent_id', '=', $forum->id);
		$table = KFactory::get($forum->getTable());
		
		if(!$table->count($query)) return;
		
		$path = $forum->path;
		foreach (KFactory::get($forum->getTable())->select($query) as $forum)
		{
			$forum->path = $path . $forum->parent_id . '/';
			$forum->save();
			
			$this->updateForumPathsRecurse($forum);
		}
	}
}