<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: kunena.php 754 2010-08-19 21:42:45Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * ComNinjaboardConverterKunena
 *
 * Imports data from Kunena.
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardConverterKunena extends ComNinjaboardConverterAbstract
{
	
	/**
	 * This converter is able to run in steps
	 *
	 * @var boolean
	 */
	public $splittable = true;

	/**
	 * Import the sample content
	 *
	 * @return $this
	 */
	public function convert()
	{
		$tables = array(
			array(
				'name' => 'forums',
				'options' => array(
					'name' => 'kunena_categories',
					'base' => 'kunena_categories',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'tbl.*',
								"'/' AS path",
								'name AS title',
								'parent AS parent_id',
								'published AS enabled',
								'numTopics AS topics',
								'numPosts AS posts',
								'id_last_msg AS last_post_id'
							))
			),
			array(
				'name' => 'topics',
				'options' => array(
					'name' => 'kunena_messages',
					'base' => 'kunena_messages',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'tbl.*',
								'catid AS forum_id',
								'topic_emoticon AS status',
								'id AS first_post_id',
								'moved AS moved_from_forum_id',
								'(SELECT SUM(hits) FROM #__kunena_messages WHERE parent = tbl.id OR id = tbl.id) AS hits',
								'(SELECT COUNT(*) FROM #__kunena_messages WHERE parent = tbl.id) AS replies',
								'(SELECT MAX(id) FROM #__kunena_messages WHERE parent = tbl.id OR id = tbl.id) AS last_post_id',
							))
							->where('parent', '=', 0)
			),
			array(
				'name' => 'posts',
				'options' => array(
					'name' => 'kunena_messages',
					'base' => 'kunena_messages',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							//->set('columns', array('!`hold` AS `enabled`'))
							->select(array(
								'tbl.*',
								'!hold AS enabled',
								'thread AS ninjaboard_topic_id',
								'topic_emoticon AS status',
								'id AS first_post_id',
								'name AS guest_name',
								'email AS guest_email',
								'userid AS created_by',
								'modified_by AS modified_by',
								'FROM_UNIXTIME(time) AS created_on',
								'ip AS user_ip',
								'FROM_UNIXTIME(modified_time) AS mofidied_on',
								'modified_reason AS edit_reason',
								'message AS text',
							))
							->join('left', 'kunena_messages_text', 'mesid = id')
			),
			array(
				'name' => 'people',
				'options' => array(
					'name' => 'kunena_users',
					'base' => 'kunena_users',
					'identity_column' => 'userid'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'*',
							))
			)
		);

		//This returns false if the import is big enough to be done in steps.
		//So we need to stop the importing in this step, in order for it to initiate
		if($this->importData($tables, 'kunena') === false) return $this;

		parent::convert();

		$this->updateForumPaths();

		return $this;
	}
	
	private function updateForumPaths()
	{
		$query = KFactory::tmp('lib.koowa.database.query')->where('parent_id', '=', 0);
		$table = KFactory::tmp('admin::com.ninjaboard.table.forums');
		
		
		//Set paths
		$table->getCommandChain()->disable();
		foreach($table->select($query, KDatabase::FETCH_ROWSET) as $forum)
		{
			$this->updateForumPathsRecurse($forum, $table);
		}
		$table->getCommandChain()->enable();
	}
	
	private function updateForumPathsRecurse($forum, $table)
	{
		$query = KFactory::tmp('lib.koowa.database.query')->where('tbl.parent_id', '=', $forum->id);

		$path = $forum->path;
		foreach ($table->select($query, KDatabase::FETCH_ROWSET) as $forum)
		{
			$forum->path = $path . $forum->parent_id . '/';
			$forum->save();

			$this->updateForumPathsRecurse($forum, $table);
		}
	}
}