<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: legacy.php 754 2010-08-19 21:42:45Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Imports data from Ninjaboard 0.5.
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardConverterLegacy extends ComNinjaboardConverterAbstract
{
	/**
	 * This converter is able to run in steps
	 *
	 * @var boolean
	 */
	public $splittable = true;

	/**
	 * Import the sample content
	 *
	 * @return $this
	 */
	public function convert()
	{
		$tables = array(
			array(
				'name' => 'forums',
				'options' => array(
					'name' => 'ninjaboard_categories_backups',
					'base' => 'ninjaboard_categories_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'(id + (SELECT MAX(id) FROM #__ninjaboard_forums_backups)) AS id',
								'(SELECT SUM(posts) FROM #__ninjaboard_forums_backups) AS posts',
								'(SELECT SUM(topics) FROM #__ninjaboard_forums_backups) AS topics',
								'(SELECT MAX(id_last_post) FROM #__ninjaboard_forums_backups) AS last_post_id',
								'name AS title',
								'published AS enabled',
								"'/' AS path"
							))
			),
			array(
				'name' => 'forums',
				'options' => array(
					'name' => 'ninjaboard_forums_backups',
					'base' => 'ninjaboard_forums_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'id',
								'description',
								'locked',
								'ordering',
								'topics',
								'posts',
								'id_last_post AS last_post_id',
								//'id AS ninjaboard_forum_id',
								'name AS title',
								'state AS enabled',
								"CONCAT('/', (id_cat + (SELECT MAX(id) FROM #__ninjaboard_forums_backups)), '/') AS path"
							))
			),
			array(
				'name' => 'usergroups',
				'options' => array(
					'name' => 'ninjaboard_groups_backups',
					'base' => 'ninjaboard_groups_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'id',
								'name AS title'
							))
			),
			
			//Permissions for admins
			$this->buildAclQueryColumn('FORUM', 3, 4),
			//Permissions for admins and mods
			$this->buildAclQueryColumn('TOPIC', 3, 2, '>'),
			$this->buildAclQueryColumn('POST', 3, 2, '>'),
			$this->buildAclQueryColumn('ATTACHMENT', 3, 2, '>'),
			//Permissions for regular users
			$this->buildAclQueryColumn('TOPIC', 1, 3, '<'),
			$this->buildAclQueryColumn('POST', 1, 3, '<'),
			$this->buildAclQueryColumn('ATTACHMENT', 1, 3, '<'),
			//Only admins got permissions beyond Has Access on forums
			$this->buildAclQueryColumn('FORUM', 1, 4, '<'),
			
			array(
				'name' => 'posts',
				'options' => array(
					'name' => 'ninjaboard_posts_backups',
					'base' => 'ninjaboard_posts_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'*',
								'date_post AS created_on',
								'date_last_edit AS modified_on',
								'id_edit_by AS modified_by',
								'ip_poster AS user_ip',
								'id_topic AS ninjaboard_topic_id',
								'id_user AS created_by',
								'IFNULL(guest.guest_name, tbl.guest_name) AS guest_name',
								"CONCAT('{\"', LOWER('CUSTOMIZATION'), '\":{\"', LOWER('ICON'), '\":\"', icon_function, '\"}}') AS params"
							))
							->join('left', 'ninjaboard_posts_guests_backups AS guest', 'guest.id_post = tbl.id')
			),
			array(
				'name' => 'ranks',
				'options' => array(
					'name' => 'ninjaboard_ranks_backups',
					'base' => 'ninjaboard_ranks_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'*',
								'name AS title',
								'min_posts AS min',
								'published AS enabled',
							))
			),
			array(
				'name' => 'subscriptions',
				'options' => array(
					'name' => 'ninjaboard_subscriptions_backups',
					'base' => 'ninjaboard_subscriptions_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'*',
								'subs_type AS subs_type_id',
								'id_user AS joomla_user_id',
								
							))
			),
			array(
				'name' => 'topics',
				'options' => array(
					'name' => 'ninjaboard_topics_backups',
					'base' => 'ninjaboard_topics_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'*',
								'id_forum AS forum_id',
								'views AS hits',
								'type AS topic_type_id',
								'id_moved AS moved_from_forum_id',
								'id_first_post AS first_post_id',
								'id_last_post AS last_post_id',
							))
			),
			array(
				'name' => 'people',
				'options' => array(
					'name' => 'ninjaboard_users_backups',
					'base' => 'ninjaboard_users_backups',
					'identity_column' => 'id'
				),
				'query' => KFactory::tmp('lib.koowa.database.query')
							->select(array(
								'*',
								'avatar_file AS avatar',
							))
			),
		);

		//This returns false if the import is big enough to be done in steps.
		//So we need to stop the importing in this step, in order for it to initiate
		if($this->importData($tables, 'ninjaboard_legacy') === false) return $this;

		parent::convert();

		return $this;
	}

	/**
	 * Creates an acl select statement
	 *
	 * @param  string $name		The name of the asset
	 * @param  int    $level	The asset level
	 * @param  int    $role		The 0.5 role level
	 * @param  string $operator	The where statemenet operator
	 * @return array	Returns select statements
	 */
	private function buildAclQueryColumn($name, $level, $role, $operator = '=')
	{
		return array(
			'name' => 'assets',
			'options' => array(
				'name' => 'ninjaboard_groups_backups',
				'base' => 'ninjaboard_groups_backups',
				'identity_column' => 'id'
			),
			'query' => KFactory::tmp('lib.koowa.database.query')
						->select(array(
							"(".rand()." + id) AS id",
							"CONCAT_WS('.', LOWER('COM_NINJABOARD'), LOWER('USERGROUP'), id, LOWER('$name')) AS name",
							"'$level' AS level"
						))
						->where('role', $operator, $role)
		);
	}
}