<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: migrator.php 754 2010-08-19 21:42:45Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard Migrator Helper
 *
 * @package Ninjaboard
 */
class ComNinjaboardHelperMigrator extends KObject
{
	
	/**
	 * The backup filepath
	 *
	 * @var string
	 */
	protected $_path = null;
	
	/**
	 * The tables that we import
	 *
	 * @var array
	 */
	protected $_filter = array(
		//'ninjaboard_categories'	=> 'ninjaboard_forums', 
		'ninjaboard_forums'		=> 'ninjaboard_forums', 
		'ninjaboard_topics'		=> 'ninjaboard_topics', 
		'ninjaboard_posts'		=> 'ninjaboard_posts'
	);
	
	/**
	 * The table maps
	 *
	 * @var array
	 */
	protected $_map = array(
		'ninjaboard_categories' => array(
			'id' => 'ninjaboard_forum_id',
			'name' => 'title',
			'published' => 'enabled',
			'ordering' => 'ordering',
			'checked_out' => 'checked_out',
			'checked_out_time' => 'checked_out_time'
		),
		'ninjaboard_forums'	 => array(
			'id' => 'ninjaboard_forum_id',
			'name' => 'title',
			'description' => 'description',
			'state' => 'enabled',
			'locked' => 'locked',
			'ordering' => 'ordering',
			'new_posts_time' => 'new_posts_time',
			'posts' => 'posts',
			'topics' => 'topics',
			'auth_view' => 'auth_view',
			'auth_read' => 'auth_read',
			'auth_post' => 'auth_post',
			'auth_reply' => 'auth_reply',
			'auth_edit' => 'auth_edit',
			'auth_delete' => 'auth_delete',
			'auth_reportpost' => 'auth_reportpost',
			'auth_sticky' => 'auth_sticky',
			'auth_lock' => 'auth_lock',
			'auth_announce' => 'auth_announce',
			'auth_vote' => 'auth_vote',
			'auth_pollcreate' => 'auth_pollcreate',
			'auth_attachments' => 'auth_attachments',
			'checked_out' => 'checked_out',
			'checked_out_time' => 'checked_out_time',
			'id_cat' => 'path',
			'id_last_post' => 'last_post_id',
			
		), 
		'ninjaboard_topics'	 => array(
			'id' => 'ninjaboard_topic_id',
			'views' => 'hits',
			'replies' => 'replies',
			'status' => 'status',
			'vote' => 'vote',
			'type' => 'topic_type_id',
			'id_moved' => 'id_moved',
			'id_forum' => 'moved_from_forum_id',
			'id_first_post' => 'first_post_id',
			'id_last_post' => 'last_post_id',
			
		), 
		'ninjaboard_posts'		 => array(
			'id' => 'ninjaboard_post_id',
			'subject' => 'subject',
			'text' => 'text',
			'date_post' => 'created_time',
			'date_last_edit' => 'modified',
			'id_edit_by' => 'modified_user_id',
			'edit_reason' => 'edit_reason',
			'enable_bbcode' => 'enable_bbcode',
			'enable_emoticons' => 'enable_emoticons',
			'notify_on_reply' => 'notify_on_reply',
			'ip_poster' => 'user_ip',
			'icon_function' => 'icon_function',
			'id_topic' => 'ninjaboard_topic_id',
			'id_forum' => 'id_forum',
			'id_parent' => 'id_parent',
			'id_user' => 'created_user_id',
			'guest_name' => 'guest_name',
			'guest_email' => 'guest_email',
			
		)
	);

	public function __construct(KConfig $options)
	{
		$path = JPATH_COMPONENT_ADMINISTRATOR.'/backups/backup.js';
		if(!isset($options->path)) $options->path = $path;
		$this->_path = $options->path;
		
		if(file_exists($this->_path)) $this->import();
	}
	
	public function import()
	{
		$data = json_decode(file_get_contents($this->_path), true);
		
		$filtered = $this->filter($data);
		
		$data = array();
		foreach($filtered as $this->name => $this->table)
		{
			$mapped = $this->map();
			$data[$this->name] = $mapped;
			$this->_import($mapped);
		}
		
		$document = KFactory::get('lib.joomla.document');
		//$document->addScriptDeclaration("console.log(".json_encode($data).")");
	}
	
	protected function _import($data)
	{
		$name  = $this->_filter[$this->name];
		$db    = KFactory::get('lib.koowa.database');
		$table = str_replace('ninjaboard_', '', $name);
		$table = KFactory::get('admin::com.ninjaboard.table.'.$table);
		
		foreach($data as $item)
		{
			//if($name == 'ninjaboard_ranks') die('<pre>'.var_export($item, true).'</pre>');
			$data = array_intersect_key($item, array_flip($table->getColumns()));
			foreach($data as $key => $val)
			{
				$vals[] = $db->quoteString($val);
				$keys[] = '`'.$key.'`';
			}
	
			$sql = 'INSERT IGNORE INTO '.$db->quoteName('#__'.$name )
				 . '('.implode(', ', $keys).') VALUES ('.implode(', ', $vals).')';

			//KFactory::get('lib.joomla.document')->addScriptDeclaration("console.log(".json_encode($sql).")");

			//Execute the query
			$db->execute($sql);
		}
	}
	
	public function filter($tables)
	{
		$data = array();
		foreach($tables as $name => $table)
		{
			if(!array_key_exists($name, $this->_filter)) continue;
			$data[$name] = $table;
		}
		return $data;
	}
	
	public function map()
	{
		$rows = array();
		foreach($this->table as $i => $row)
		{
			foreach($row as $key => $val)
			{
				if(!isset($this->_map[$this->name][$key])) continue;
				$key = $this->_map[$this->name][$key];
				$rows[$i][$key] = $val;
			}
			
		}
		return $rows;
	}
}