<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: forums.php 833 2010-09-08 22:20:02Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard Forums model
 *
 * Fetches forums, and all that comes with the forums
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardModelForums extends ComNinjaModelTable
{
	/**
	 * This is for the frontend router
	 *
	 * @var boolean
	 */
	public $sef = true;

	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		parent::__construct($config);
		
		// Set the state
		// @TODO we need to look at what states we can purge, is getting messy here
		$this->_state
			->insert('acl'       , 'cmd', 'auth_view')
			->insert('exclude', 'int', 0)
			->insert('hierarchy', 'boolean', false)
			->insert('enabled', 'boolean', false)
			->insert('recurse', 'boolean', false)
			->insert('flat', 'boolean', false)
			->insert('path', 'int')
			->insert('parent'   , 'int')
			->insert('level'   , 'int', 1)
			->insert('levels', 'int')
			->insert('sort', 'cmd', 'path_sort');
	}
	
	protected function _buildQueryJoins(KDatabaseQuery $query)
	{
		$query
				->join('left', 'ninjaboard_posts AS last_post', 'last_post.ninjaboard_post_id = tbl.last_post_id')
				->join('left', 'ninjaboard_topics AS topic', 'last_post.ninjaboard_topic_id = topic.ninjaboard_topic_id')
				->join('left', 'ninjaboard_posts AS first_post', 'first_post.ninjaboard_post_id = topic.first_post_id')
				->join('left', 'users AS usr', 'usr.id = last_post.created_user_id');
	}
	
	protected function _buildQueryColumns(KDatabaseQuery $query)
	{
		parent::_buildQueryColumns($query);

		/* TODO finish this function and remove the placeholder */
		$query->select('first_post.subject')
			  ->select('IFNULL(usr.username, \''.JText::_('Anonymous').'\') AS last_post_username')
			  ->select('last_post.created_user_id');
		
		$query->select('tbl.last_post_id')
			  ->select('last_post.created_time AS last_post_date')
			  ->select('last_post.ninjaboard_topic_id AS last_topic_id')
			  ->select('last_post.ninjaboard_post_id AS last_post_id');

		//Build the query for fetching the permissions
		$this->_buildPermissionsQuery($query);
	}
	
	protected function _buildQueryOrder(KDatabaseQuery $query)
    {
    	$sort      = $this->_state->sort;
       	$direction  = strtoupper($this->_state->direction);

		if(!$sort or $sort == 'path_sort') {
			//@TODO we no longer need this CONCAT part
			//$query->select("CONCAT(tbl.path, tbl.ninjaboard_forum_id, '/') AS path_ordering");
			$query->order('path_sort', 'asc');
		} elseif($sort) {
			//Our model don't support DESC yet
			//$query->order($sort, $direction);
			$query->order($sort, 'asc');
    	}
    }
	
	/**
	 * Builds the query that gets the permission level per object
	 *
	 * @param KDatabaseQuery $query
	 */
	protected function _buildPermissionsQuery(KDatabaseQuery $query)
	{
		if(isset($this->_permissions))	return;
		else							$this->_permissions = true;
	
		$me = KFactory::get('admin::com.ninjaboard.model.people')->getMe();

		$objects = KFactory::get('admin::com.ninjaboard.permissions')->getObjects();

		if($me->gid == 25)
		{
			foreach($objects as $object)
			{
				$query->select('3 AS '.$object.'_permissions');
			}

			return $this;
		}
		
		$groups		= '('.$me->ninjaboard_usergroup_id.')';
	
		
		foreach ($objects as $object) 
		{			
			$alias		= $object.'_permissions';
			$ifnull		= (int)$me->{$object.'_permissions'};

			$query->select("IFNULL((".
				"SELECT level ".
				"FROM #__ninjaboard_assets ".
				"WHERE name RLIKE CONCAT('COM_NINJABOARD[.]FORUM[.]', CAST(tbl.ninjaboard_forum_id AS CHAR), '[.]$groups"."[.]".strtoupper($object)."') ".
				"ORDER BY level DESC ".
				"LIMIT 1 "
			."), $ifnull) AS $object".'_permissions');
		}
		
		//WHERE clause using a subquery to check the forum permission level
		//@TODO figure out how this can be optimized
		$object		= 'forum';
		$ifnull		= (int)$me->{$object.'_permissions'};

		$query->where("IFNULL((".
			"SELECT level ".
			"FROM #__ninjaboard_assets ".
			"WHERE name RLIKE CONCAT('COM_NINJABOARD[.]FORUM[.]', CAST(tbl.ninjaboard_forum_id AS CHAR), '[.]$groups"."[.]FORUM') ".
			"ORDER BY level DESC ".
			"LIMIT 1 "
		."), $ifnull)", '>', 0);
	}
	
	protected function _buildQueryWhere(KDatabaseQuery $query)
	{
		parent::_buildQueryWhere($query);

		// If we have an id, we shouldn't add other where statements
		// @TODO check if it's safe to do this
		if($this->_state->id) return;

		if($this->_state->enabled)
		{
			$query->where('tbl.enabled', '=', 1, 'and');
		}
		if($this->_state->recurse && $this->_state->path && !$this->_state->flat)
		{
			//$query->where('tbl.path', 'like', '%'.$this->_state->path);
			//$query->where('tbl.path', 'like', '%/'.$this->_state->path.'/%');
			$query->where('tbl.path', 'like', '%/'.$this->_state->path.'/');
		}
		else if($this->_state->recurse && !$this->_state->flat)
		{
			$query->where('tbl.path', '=', '/');
		}
		
		if($this->_state->levels !== null)
		{
			$level  = $this->_state->level;
			$levels = $this->_state->levels;
			$query->where('tbl.level', '>=', $level, 'and')->where('tbl.level', '<', $levels + $level);
		}
		
		//Build the query for fetching the permissions
		$this->_buildPermissionsQuery($query);
	}
	
	public function getList()
    {
	    if(!isset($this->_list))
	    {
	        parent::getList();

	        if($this->getTotal() < 1) return $this->_list = array();
	        
	        if($this->_state->levels)
	        {
	        	$copied = array();

	        	$list = $this->_list->getIterator()->getArrayCopy();
	        	$table = KFactory::get($this->getTable());
	        	//usort($list, array($this, 'sort'));
	        	
	        	foreach($list as $row)
	        	{
	        		if($row->level < 2) continue;

	        		$parent = end(array_filter(explode('/', $row->path)));
	        		
	        		//If parent isn't enabled, don't try to find it
	        		if(!$table->count(array('id' => $parent, 'enabled' => 1))) continue;
	        		
	        		$parent = $this->_list->find($parent);

	        		//@TODO perhaps move this into the table object instead, as it's a virtual property
	        		if(!isset($parent->subforums)) $parent->subforums = array();

	        		$subforums = $parent->subforums;
	        		$subforums[] = $row;
	        		$parent->subforums = $subforums;

	        		$copied[] = $row;
	        	}

	        	// Remove copied forums from main list
	        	foreach($copied as $row)
	        	{
	        		$this->_list->removeRow($row);
	        	}
	        }
	        
	        if($this->_state->recurse) $this->_total = count($this->_list);		
		}
		
		return $this->_list;
    }
    
    public function sort($a, $b)
    {
    	return $b->level - $a->level;
    }

	/**
	 * Get a list over parent forums, including current forum
	 *
	 * @author	Stian Didriksen <stian@ninjaforge.com>
	 * @return	KDatabaseRowsetInterface
	 */
	public function getListWithParents()
	{
		$database = KFactory::get('lib.koowa.database');
		$table = KFactory::get($this->getTable());
		$query = KFactory::tmp('lib.koowa.database.query')
					->select('path')
					->where('ninjaboard_forum_id', '=', $this->_state->id);
		
		$path = $table->select($query, KDatabase::FETCH_FIELD);
		
		$ids = array_merge(array($this->_state->id), array_filter(explode('/', $path)));
		
		return $table->select($ids, KDatabase::FETCH_ROWSET);
	}
}