<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: people.php 810 2010-09-01 13:57:16Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard People model
 *
 * Stores the user, with avatar, forum signature and such
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardModelPeople extends KModelTable
{
	/**
	 * The current active user
	 *
	 * @var KDatabaseRow
	 */
	protected $_me;

	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $options)
	{
		parent::__construct($options);
				
		// Set the state
		$this->_state
			->insert('user', 'boolean', true);
	}
	
	protected function _buildQueryColumns(KDatabaseQuery $query)
	{
		parent::_buildQueryColumns($query);
		
		$query
				->select(array(
					'user.*',
					'IFNULL(tbl.posts, 0) AS posts'
				))
				//Reduce this to one query
				->select(array(
					'(SELECT rank_file FROM #__ninjaboard_ranks WHERE IFNULL(tbl.posts, 0) >= min ORDER BY min DESC LIMIT 0, 1) AS rank_icon',
					'(SELECT title FROM #__ninjaboard_ranks WHERE IFNULL(tbl.posts, 0) >= min ORDER BY min DESC LIMIT 0, 1) AS rank_title',
					//'rank.rank_file AS rank_icon',
					//'rank.title AS rank_title',
				));
	}
	
	protected function _buildQueryJoins(KDatabaseQuery $query)
	{
		$query
				//->join('left', 'ninjaboard_people AS person', 'tbl.id = person.ninjaboard_person_id');
				->join('left', 'users AS user', 'user.id = tbl.ninjaboard_person_id');
				//->join('left', 'ninjaboard_ranks AS rank', 'IFNULL(person.posts, 0) >= rank.min')
				//->order('rank.min', 'DESC');
		
		$query
			->join('left', 'ninjaboard_joomla_user_group_maps AS joomla_map', 'joomla_map.joomla_gid = user.gid')
			->select("IFNULL((SELECT GROUP_CONCAT(ninjaboard_map.ninjaboard_user_group_id SEPARATOR '|') FROM #__ninjaboard_user_group_maps AS ninjaboard_map WHERE ninjaboard_map.joomla_user_id = user.id), joomla_map.ninjaboard_gid) AS ninjaboard_usergroup_id");
		
		parent::_buildQueryJoins($query);
	}

	/**
	 * Builds a WHERE clause for the query
	 * Without tbl. alias, as it caused errors frontend when posting
	 */
	protected function _buildQueryWhere(KDatabaseQuery $query)
	{
		if($search = $this->_state->search)
		{
			$query
				  ->where('tbl.name', 'LIKE',  '%'.$search.'%', 'or')
				  ->where('tbl.username', 'LIKE',  '%'.$search.'%', 'or')
				  ->where('tbl.email', 'LIKE',  '%'.$search.'%', 'or');
		}
		
		parent::_buildQueryWhere($query);
	}
	
	/**
	 * Builds the query that gets the joomla groupname
	 *
	 * @param KDatabaseQuery $query
	 */
	protected function _buildAroQuery(KDatabaseQuery $query)
	{
		$query
			->select(array(
				'user.gid AS joomla_usergroup_id',
				'core_acl_aro_groups.value AS joomla_usergroup_title'
			))
			->join('inner', 'core_acl_aro AS core_acl_aro', 'core_acl_aro.value = user.id')
			->join('inner', 'core_acl_groups_aro_map AS core_acl_groups_aro_map', 'core_acl_groups_aro_map.aro_id = core_acl_aro.id')
			->join('inner', 'core_acl_aro_groups AS core_acl_aro_groups', 'core_acl_aro_groups.id = core_acl_groups_aro_map.group_id');
	}
	
	/**
	 * Builds the query that gets the permission level per object for a person
	 *
	 * @param KDatabaseQuery $query
	 */
	protected function _buildPersonPermissionsQuery(KDatabaseQuery $query)
	{
		$query
			->join('left', 'ninjaboard_joomla_user_group_maps AS joomla_map', 'joomla_map.joomla_gid = user.gid')
			->select("IFNULL((SELECT GROUP_CONCAT(ninjaboard_map.ninjaboard_user_group_id SEPARATOR '|') FROM #__ninjaboard_user_group_maps AS ninjaboard_map WHERE ninjaboard_map.joomla_user_id = user.id), joomla_map.ninjaboard_gid) AS ninjaboard_usergroup_id");

		$objects = KFactory::get('admin::com.ninjaboard.permissions')->getObjects();

		foreach ($objects as $object) 
		{
			$alias = 'usergroup_'.$object.'_permissions';
			$query->join(
				'left',
				"ninjaboard_assets AS $alias",
				"$alias.name LIKE CONCAT_WS('.', 'COM_NINJABOARD', 'USERGROUP', CAST(joomla_map.ninjaboard_gid AS CHAR), '".strtoupper($object)."')"
			)->select($alias.'.level AS '.$alias);
			
			$alias		= 'person_'.$object.'_permissions';
			$groups		= "(SELECT CONCAT('(', GROUP_CONCAT(map_$alias.ninjaboard_user_group_id SEPARATOR '|'), ')') FROM #__ninjaboard_user_group_maps AS map_$alias WHERE map_$alias.joomla_user_id = user.id)";

			$query->select("IFNULL((".
				"SELECT $alias.level ".
				"FROM #__ninjaboard_assets AS $alias ".
				"WHERE $alias.name RLIKE CONCAT('COM_NINJABOARD[.]USERGROUP[.]', CAST($groups AS CHAR), '[.]".strtoupper($object)."') ".
				//"WHERE $alias.name RLIKE $pattern"
				"ORDER BY $alias.level DESC ".
				"LIMIT 1 "
			."), usergroup_$object"."_permissions.level) AS $object".'_permissions');
		}
	}
	
	/**
	 * Builds the query that gets the permission level per object for a guest
	 *
	 * @param KDatabaseQuery $query
	 */
	protected function _buildQuestPermissionsQuery(KDatabaseQuery $query)
	{
		$query
			->select("joomla_map.ninjaboard_gid AS ninjaboard_usergroup_id");

		$objects = KFactory::get('admin::com.ninjaboard.permissions')->getObjects();
		foreach ($objects as $object) 
		{
			$alias = $object.'_permissions';
			$query->join(
				'left',
				"ninjaboard_assets AS $alias",
				"$alias.name LIKE CONCAT_WS('.', 'COM_NINJABOARD', 'USERGROUP', CAST(joomla_map.ninjaboard_gid AS CHAR), '".strtoupper($object)."')"
			)->select($alias.'.level AS '.$alias);
		}
	}

	/**
	 * Gets the current joomla user with data like permissions
	 *
	 * If the user is a guest, we still got some guest data to load
	 *
	 * @return KDatabaseRow
	 */
	public function getMe()
	{
		if (!isset($this->_me))
        {
        	$user = KFactory::get('lib.joomla.user');
        	$id   = $user->id;

        	$table  = KFactory::get($this->getTable());
			$query = $table->getDatabase()->getQuery();
			
			if($user->guest) {
				$query
					->select(array(
						"LOWER(CONCAT('BLUE/0', '.', 'PNG')) AS rank_icon",
						"'".JText::_('Guest')."' AS rank_title",
					))
					->from('ninjaboard_joomla_user_group_maps AS joomla_map')
					->where('joomla_map.joomla_gid', '=', 0)
					->where('joomla_map.ninjaboard_gid', '!=', 0)
					->limit(1)
					;
				
								
				//Build the query for fetching the permissions
				$this->_buildQuestPermissionsQuery($query);
			} else {
				$query
					->select(array(
						'person.*',
						'user.*',
						'rank.rank_file AS rank_icon',
						'rank.title AS rank_title',
					))
					->from('users AS user')
					->join('left', 'ninjaboard_people AS person', 'user.id = person.ninjaboard_person_id')
					->join('left', 'ninjaboard_ranks AS rank', 'person.posts >= rank.min')
					->where('user.id', '=', $id)
					->order('rank.min', 'DESC')
					;
					
				//Build the complex joomla query for getting the core usergroup
				$this->_buildAroQuery($query);
				
				//Build the query for fetching the permissions
				$this->_buildPersonPermissionsQuery($query);
			}

			$this->_me = $table->select($query, KDatabase::FETCH_ROW);
//			die('<pre>'.print_r($this->_me, true).'</pre>');
		}
		
		return $this->_me;
	}
}