<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: topics.php 754 2010-08-19 21:42:45Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard Topics model
 *
 * Fetches forums, and such
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardModelTopics extends KModelTable
{

	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		parent::__construct($config);
				
		$this->_state
						->insert('forum', 'int')
						->insert('at'	, 'int', false);
	}

	protected function _buildQueryJoins(KDatabaseQuery $query)
	{
		parent::_buildQueryJoins($query);

		$query->join('LEFT', 'ninjaboard_posts AS first_post', 'first_post.ninjaboard_post_id = tbl.first_post_id')
				->join('LEFT', 'ninjaboard_posts AS last_post', 'last_post.ninjaboard_post_id = tbl.last_post_id')
				->join('LEFT', 'users AS first_post_user', 'first_post_user.id = first_post.created_user_id')
				->join('LEFT', 'users AS last_post_user', 'last_post_user.id = last_post.created_user_id');
	}
	
	protected function _buildQueryWhere(KDatabaseQuery $query)
	{
		parent::_buildQueryWhere($query);

		// @TODO commeted out until we figure out why it breaks the posting
		//$query->where('first_post.subject', '!=', '', 'AND');
		if($this->_state->forum) $query->where('tbl.forum_id', '=', $this->_state->forum);
		
		if($this->_state->at) $query->where('first_post.created_user_id', '=', $this->_state->at);
		
		//Build the query for fetching the permissions
		$this->_buildPermissionsQuery($query);
	}
	
	protected function _buildQueryColumns(KDatabaseQuery $query)
	{
		parent::_buildQueryColumns($query);

		$query
				->select(array(
					'tbl.ninjaboard_topic_id',
					'tbl.replies',
					'tbl.status',
					'tbl.vote',
					'tbl.topic_type_id',
					'tbl.moved_from_forum_id',
					'tbl.forum_id',
					'tbl.first_post_id',
					'tbl.last_post_id',
					'tbl.hits',
					'tbl.enabled',
					'tbl.sticky',
					'tbl.locked',
					'tbl.resolved'
				))
				->select('first_post.subject')
				->select('first_post.subject AS title')
				->select('last_post.subject AS last_post_subject')
				->select('first_post.created_time AS first_post_date')
				->select('last_post.created_user_id')
				->select('last_post.created_time AS last_post_date')
				->select('IFNULL(first_post_user.username, \''.JText::_('Anonymous').'\') AS first_post_username')
				->select('IFNULL(last_post_user.username, \''.JText::_('Anonymous').'\') AS last_post_username');
				
		//Build the query for fetching the permissions
		$this->_buildPermissionsQuery($query);
	}
	
	/**
	 * Builds the query that gets the permission level per object
	 * 
	 * It's called during both buildQueryColumns and buildQueryWhere, but only run once.
	 * This is to ensure that it's always executed.
	 *
	 * @param KDatabaseQuery $query
	 */
	protected function _buildPermissionsQuery(KDatabaseQuery $query)
	{
		if(isset($this->_permissions))	return;
		else							$this->_permissions = true;
	
		$me = KFactory::get('admin::com.ninjaboard.model.people')->getMe();
		//@TODO make the acl objects array dynamic and not hard coded
		$objects = array('topic');
		
		if($me->gid == 25)
		{
			foreach($objects as $object)
			{
				$query->select('3 AS '.$object.'_permissions');
			}

			return $this;
		}
		
		$groups		= '('.$me->ninjaboard_usergroup_id.')';
	
		
		foreach ($objects as $object) 
		{			
			$alias		= $object.'_permissions';
			$ifnull		= (int)$me->{$object.'_permissions'};

			$query->select("IFNULL((".
				"SELECT level ".
				"FROM #__ninjaboard_assets ".
				"WHERE name RLIKE CONCAT('COM_NINJABOARD[.]FORUM[.]', CAST(tbl.forum_id AS CHAR), '[.]$groups"."[.]".strtoupper($object)."') ".
				"ORDER BY level DESC ".
				"LIMIT 1 "
			."), $ifnull) AS $object".'_permissions');
		}
		
		//WHERE clause using a subquery to check the forum permission level
		//@TODO figure out how this can be optimized
		$object		= 'topic';
		$ifnull		= (int)$me->{$object.'_permissions'};

		$query->where("IFNULL((".
			"SELECT level ".
			"FROM #__ninjaboard_assets ".
			"WHERE name RLIKE CONCAT('COM_NINJABOARD[.]FORUM[.]', CAST(tbl.forum_id AS CHAR), '[.]$groups"."[.]TOPIC') ".
			"ORDER BY level DESC ".
			"LIMIT 1 "
		."), $ifnull)", '>', 0);
	}
    
    /**
     * Get a topic row object
     *
     * @return KDatabaseRowset
     */
    public function getItem()
    {
    	if (!isset($this->_item))
    	{
	        parent::getItem();
	        
	        //@TODO acl workaround
	        if(!$this->_item->forum_id && $this->_state->forum) $this->_item->forum_id = $this->_state->forum;
		}

        return $this->_item;
    }
}