<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: users.php 810 2010-09-01 13:57:16Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard Users model
 *
 * Gets the user, with avatar, forum signature and such
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardModelUsers extends KModelTable
{
	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		parent::__construct($config);
				
		// Set the state
		$this->_state
			->insert('id', 'int', true)
			->insert('usergroup', 'int')
			->insert('gid'		, 'boolean', false)
			->insert('new'		, 'boolean', false);
	}

	protected function _buildQueryJoins(KDatabaseQuery $query)
	{
		$query->join('left', 'ninjaboard_people AS person', 'person.ninjaboard_person_id = tbl.id')
			  ->join('left', 'ninjaboard_joomla_user_group_maps AS jmp', 'jmp.joomla_gid = tbl.gid')
			  ->join('inner', 'core_acl_aro AS aro', 'aro.value = tbl.id')
			  ->join('inner', 'core_acl_groups_aro_map AS gam', 'gam.aro_id = aro.id')
			  ->join('inner', 'core_acl_aro_groups AS grp', 'grp.id = gam.group_id');

		$query
			->join('left', 'ninjaboard_joomla_user_group_maps AS joomla_map', 'joomla_map.joomla_gid = tbl.gid')
			->select("IFNULL((SELECT GROUP_CONCAT(ninjaboard_map.ninjaboard_user_group_id SEPARATOR '|') FROM #__ninjaboard_user_group_maps AS ninjaboard_map WHERE ninjaboard_map.joomla_user_id = tbl.id), joomla_map.ninjaboard_gid) AS ninjaboard_usergroup_id");

		parent::_buildQueryJoins($query);
	}
	
	protected function _buildQueryColumns(KDatabaseQuery $query)
	{
		//Get the default gid for users that's not mapped
		$gid	= (int)KFactory::get('admin::com.ninjaboard.model.joomla_usergroup_maps')->getGuest()->gid;

		$query
			  ->select('tbl.*')
			  ->select('grp.name AS groupname')
			  ->select("IFNULL(jmp.ninjaboard_gid, $gid) AS group_id")
			  ->select(array(
			  	'IFNULL(person.posts, 0) AS posts',
			  	//@TODO All this just to avoid quoting
			  	'IFNULL(person.avatar, CONCAT(LOWER(\'/MEDIA/COM_NINJABOARD/IMAGES/AVATAR\'), LOWER(\'.PNG\'))) AS avatar',
			  	'person.signature AS signature'
			  ))
			  //Reduce this to one query
			  ->select(array(
			  	'(SELECT rank_file FROM #__ninjaboard_ranks WHERE IFNULL(person.posts, 0) >= min ORDER BY min DESC LIMIT 0, 1) AS rank_icon',
			  	'(SELECT title FROM #__ninjaboard_ranks WHERE IFNULL(person.posts, 0) >= min ORDER BY min DESC LIMIT 0, 1) AS rank_title',
			  	//'rank.rank_file AS rank_icon',
			  	//'rank.title AS rank_title',
			  ));
		
		parent::_buildQueryColumns($query);
	}

	/**
	 * Filter list based on search keyword
	 *
	 * @author Stian Didriksen <stian@ninjaforge.com>
	 * @param KDatabaseQuery $query
	 * @return $this
	 */
	protected function _buildQueryWhere(KDatabaseQuery $query)
	{
		// exclude any child group id's for this user
		if($this->_state->gid)
		{
			$gid  = KFactory::get('lib.joomla.user')->get('gid');
			//$gids = KFactory::get('lib.joomla.authorization')->get_group_children($gid, 'ARO', 'RECURSE');
		
			if (is_array($gids) && count($gids) > 0) {
				//$query->where('tbl.gid', 'NOT IN', $gids);
			}
		}
       	if($search = $this->_state->search)
		{
			$query
				  ->where('tbl.name', 'LIKE',  '%'.$search.'%', 'or')
				  ->where('tbl.username', 'LIKE',  '%'.$search.'%', 'or')
				  ->where('tbl.email', 'LIKE',  '%'.$search.'%', 'or');
		}
		
		$group = $this->_state->usergroup;
		if($group)
		{
			//$query->where('tbl.gid', '=',  (int)$group, 'and');
		}

		parent::_buildQueryWhere($query);
	}
}