<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: forums.php 827 2010-09-06 22:47:26Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link	 	http://ninjaforge.com
 */

/**
 * Ninjaboard forums table
 * 
 * @author Stian Didriksen <stian@ninjaforge.com>
 */
class ComNinjaboardTableForums extends KDatabaseTableDefault
{
	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		$config->name    = 'ninjaboard_forums';
		$config->base    = 'ninjaboard_forums';
		
		$nestable			  = KFactory::tmp('admin::com.ninja.behavior.nestable');
		//$configurable		  = KFactory::tmp('admin::com.ninjaboard.behavior.configurable');
		$config->behaviors = array($nestable/*, $configurable*/);
		$config->filters =  array(
			'params' => 'json'
		);
		
		parent::__construct($config);
		
		//@TODO change the default value to '/' in the table schema later.
		$fields = $this->getColumns();
		$fields['path']->default = '/';		
	}
	
	/**
	 * Table insert method
	 *
	 * @param  object	A KDatabaseRow object
	 * @return boolean  TRUE if successfull, otherwise false
	 */
	public function insert( KDatabaseRowInterface $row )
	{
		$this->_setLevel($row);

		//This is because we need behavior.nestable to set the right path, before we set the path_sort
		//row->getModified() returns a blank array after parent::update have run
		$modified = $row->getModified();
		
		$result = parent::insert($row);

		$this->_setSort($row, $modified);

		return $result;
	}

	/**
	 * Table update method
	 *
	 * @param  object	A KDatabaseRow object
	 * @return boolean  TRUE if successfull, otherwise false
	 */
	public function update( KDatabaseRowInterface $row)
	{
		$this->_setLevel($row);

		//This is because we need behavior.nestable to set the right path, before we set the path_sort
		//row->getModified() returns a blank array after parent::update have run
		$modified = $row->getModified();

		$result = parent::update($row);
		
		$this->_setSort($row, $modified);

		return $result;
	}
	
	/**
	 * Table maintenance method
	 *
	 * Runs a maintenance procedure
	 */
	public function maintenance()
	{
		foreach($this->select(array('level' => 0)) as $forum)
		{
			$this->_setLevel($forum);
			$forum->save();
		}
		
		if($this->count(array('ordering' => 1)) > 1)
		{
			foreach($this->select(array('ordering' => 1), KDatabase::FETCH_ROWSET) as $id => $forum)
			{
				$forum->ordering = $id;
				$forum->save();
			}
		}

		$query = KFactory::tmp('lib.koowa.database.query')
															->select('*')
															->select("CONCAT(path, ninjaboard_forum_id, '/') AS path_temp")
															->where('path_sort', '=', '', 'or')
															->where('path_sort_title', '=', '', 'or')
															->where('path_sort_ordering', '=', '', 'or')
															->order('path_temp', 'asc');
		foreach($this->select($query, KDatabase::FETCH_ROWSET) as $forum)
		{
			$this->_setSort($forum, array('path'));
		}
		
		foreach($this->select(array('alias' => '')) as $forum)
		{
			$forum->alias = KFactory::tmp('lib.koowa.filter.slug')->sanitize($forum->title);
			$forum->save();
		}

		return $this;
	}
	
	/**
	 * Sets row level based on path
	 *
	 * @param  object	A KDatabaseRow object
	 * @return KDatabaseRowset
	 */
	private function _setLevel(KDatabaseRowInterface $row)
	{
		$row->level = substr_count($row->path, '/');
	
		return $row;
	}
	
	/**
	 * Sets the sort columns
	 *
	 * @param  object	A KDatabaseRow object
	 * @param  array	Array over modified columns
	 * @return KDatabaseRowset
	 */
	private function _setSort(KDatabaseRowInterface $row, array $modified = array())
	{
		//Only run if title, path or ordering is changed
		if(
			!in_array('title', $modified) &&
			!in_array('path', $modified) &&
			!in_array('ordering', $modified
		)) {
			return;
		}
		//Prevents eternal recursing
		if(
			in_array('path_sort', $modified) ||
			in_array('path_sort_title', $modified) ||
			in_array('path_sort_ordering', $modified
		)) {
			return;
		}
		
		$parents	= array_filter(explode('/', $row->path));
		$rows		= array($row);
		$ids		= array('/'.$row->id.'/');
		$titles		= array('/'.$row->title.'/');
		$orderings	= array('/'.$row->ordering.'/');
		
		$query = $this->getDatabase()->getQuery()
													->select('*')
													->select("CONCAT(path, ninjaboard_forum_id, '/') AS path_temp")
													->where('path', 'LIKE', '%/'.$row->id.'/%', 'or')
													->order('path_temp', 'asc');
		if($parents) $query->where('ninjaboard_forum_id', 'in', $parents, 'or');
		foreach($this->select($query, KDatabase::FETCH_ROWSET) as $child)
		{
			$rows[]			= $child;
			$ids[]			= '/'.$child->id.'/';
			$titles[]		= '/'.$child->title.'/';
			$orderings[]	= '/'.$child->ordering.'/';
		}
		
		foreach($rows as $item)
		{
			$item->path_sort			= $item->path.$item->id.'/';
			$item->path_sort_title		= str_replace($ids, $titles, $item->path_sort);
			$item->path_sort_ordering	= str_replace($ids, $orderings, $item->path_sort);
			
			$item->save();
		}
	}

	/**
	 * Table delete method
	 *
	 * @param  object	A KDatabaseRow object
	 * @return boolean  TRUE if successfull, otherwise false
	 */
	public function delete( KDatabaseRowInterface $row )
	{
		
		
		parent::delete($row);
		
		
	}
}