<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: person.php 903 2010-09-16 00:43:28Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard Person Controller
 *
 * @package Ninjaboard
 */
class ComNinjaboardControllerPerson extends ComNinjaboardControllerDefault
{
	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		//When no id is set in the url, then we should assume the user wants to see his own profile
		$me		= KFactory::get('site::com.ninjaboard.model.people')->getMe();
		$config->request->append(array(
			'user' => true,
			'id'   => $me->id
		));
		
		parent::__construct($config);

		$this->registerFunctionBefore(array('edit', 'apply', 'save'), 'checkPermissions');
		$this->registerFunctionAfter(array('add', 'edit'), 'setAvatar');
	}

	public function checkPermissions()
	{
		$me		= KFactory::get($this->getModel())->getMe();
		$person	= KFactory::get($this->getModel())->getItem();
		
		/*
		if($me->id !== $person->id) {
			$this->execute('cancel');

			return false;
		}
		//*/
		
		return $me->id === $person->id;
	}
	
	public function setAvatar(KCommandContext $context)
	{
		//If nothing is uploaded, don't execute
		if(!KRequest::get('files.avatar.name', 'raw')) return;

		//Prepare MediaHelper
		JLoader::register('MediaHelper', JPATH_ROOT.'/components/com_media/helpers/media.php');

		$data			= $context['result'];
		$person			= KFactory::get($this->getModel())->getItem();
		$error			= null;
		$errors			= array();
		$identifier		= $this->getIdentifier();
		$name			= $identifier->type.'_'.$identifier->package;
		$relative		= '/media/'.$name.'/images/avatars/'.$person->id.'/';
		$absolute		= JPATH_ROOT.$relative;
		$attachments	= array();
		
		
		$avatar = KRequest::get('files.avatar', 'raw');
		if(!MediaHelper::canUpload($avatar, $error)) {
			$message = JText::_("%s failed to upload because %s");
			return JError::raiseWarning(21, sprintf($message, $avatar['name'], lcfirst($error)));
		}
		if(!MediaHelper::isImage($avatar['name'])) {
			$message = JText::_("%s failed to upload because it's not an image.");
			return JError::raiseWarning(21, sprintf($message, $avatar['name']));
		}
		
		$this->params = KFactory::get('admin::com.ninjaboard.model.settings')->getParams();
		$params = $this->params['avatar_settings'];
		$maxSize = (int) $params['upload_size_limit'];
		if ($maxSize > 0 && (int) $avatar['size'] > $maxSize)
		{
			$message = JText::_("%s failed uploading because it's too large.");
			return JError::raiseWarning(21, sprintf($message, $avatar['name']));
		}
			

		$upload = JFile::makeSafe(uniqid(time())).'.'.JFile::getExt($avatar['name']);
		JFile::upload($avatar['tmp_name'], $absolute.$upload);

		$person->avatar = $relative.$upload;
		$person->save();
		
		//@TODO we shouldn't clear all cache, only the cache for this user
		if(JFolder::exists(JPATH_ROOT.'/cache/com_ninjaboard/avatars')) JFolder::delete(JPATH_ROOT.'/cache/com_ninjaboard/avatars');
	}

	/**
	 * Read action, creates an row if it don't already exist
	 */
	protected function _actionRead()
	{
		$request	= $this->getRequest();
		$row		= parent::_actionRead();
		
		if(!$row->id && $request->id) {
			//Check that the person exists, before creating Ninjaboard record
			$exists = KFactory::get('site::com.ninjaboard.model.users')->id($request->id)->getTotal() > 0;
			if(!$exists) {
				JError::raiseError(404, JText::_('Person not found.'));
			}
		
			$row->id = $request->id;
			$row->save();
			
			//In order to get the data from the jos_users table, we need to rerun the query by getting a fresh row and setting the data
			$new = KFactory::tmp($this->getModel())->id($row->id)->getItem();
			$row->setData($new->getData());
		}
		
		//an id is absolutely required
		if(!$row->id && !KFactory::get('lib.joomla.user')->guest) {
			JError::raiseError(404, JText::_('Person not found.'));
			
		}
		
		if(isset($request->layout) && $request->layout == 'form')
		{
			$me		= KFactory::get($this->getModel())->getMe();
			if($me->id && $row->id !== $me->id)
			{
				$message = "You can't edit other users than yourself.";
				KFactory::get('lib.koowa.application')
					->redirect(JRoute::_('&view=person&id='.$row->id.'&layout='), JText::_($message), 'error');
				return $row;
			}
		}

		return $row;
	}

	/*
	 * Generic cancel action
	 *
	 * @return 	void
	 */
	protected function _actionCancel()
	{
		$person	= KFactory::get($this->getModel())->getItem();
		
		$this->_redirect = 'view=person&id='.$person->id;
	}

	/**
	 * Apply action, workaround for redirects
	 */
	protected function _actionApply($data)
	{
		$result = parent::_actionApply($data);
	
		$this->_redirect = 'view=person&id='.$result->id.'&layout=default';
		
		return $result;
	}

	/*
	 * Empty delete action
	 *
	 * Users can't be deleted
	 *
	 * @return 	void
	 */
	protected function _actionDelete()
	{
		return false;
	}
}