<?php defined( 'KOOWA' ) or die( 'Restricted access' );
/**
 * @version		$Id: post.php 886 2010-09-15 09:09:16Z stian $
 * @category	Ninjaboard
 * @copyright	Copyright (C) 2007 - 2010 NinjaForge. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://ninjaforge.com
 */

/**
 * Ninjaboard Post Controller
 *
 * @package Ninjaboard
 */
class ComNinjaboardControllerPost extends ComNinjaboardControllerDefault
{
	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		$config->append(array(
			'request' => array(
				'layout' => 'form'
			)
		));
	
		parent::__construct($config);

		//Register validation event
		//@TODO we shouldn't have to attach to the save and apply events. But KControllerView expects 'edit' to succeed.
		$this->registerFunctionBefore(array('add', 'edit', 'save', 'apply'), 'validate');

		$this->registerFunctionAfter(array('add', 'edit'), 'setAttachments');
		
		//Delete related event handlers
		$this->registerFunctionBefore('delete', 'canDelete');
		$this->registerFunctionAfter('delete', 'cleanupDelete');
	}

	/**
	 * Validates the entered data
	 *
	 * @param	KCommandContext	The context of the event
	 * @return	boolean			True when valid, false when invalid to stop the action from executing
	 */
	public function validate(KCommandContext $context)
	{
		$data 	 = $context->data;
		$request = $this->getRequest();
		
		//If there's a topic id, then we're replying or editing
		if(!empty($data->ninjaboard_topic_id) && empty($data->subject)) {
			$topic = KFactory::tmp('site::com.ninjaboard.model.topics')
																		->id($data->ninjaboard_topic_id)
																		->getItem();
			//If the id is set, then we're editing
			if(isset($request->id)) {
				//If this post is the topic starter, it can't be without a subject or body
				if($topic->first_post_id === $request->id) {
					JError::raiseWarning(21, JText::_('Topics cannot be without a subject.'));
					$this->execute('cancel');
					return false;
				} 
			}
		} elseif(empty($data->subject)) {
			JError::raiseWarning(21, JText::_('Topics cannot be without a subject.'));
			$this->execute('cancel');
			return false;
		}
		
		if(empty($data->text)) {
			JError::raiseWarning(21, JText::_('Posts cannot be without text.'));
			$this->execute('cancel');
			return false;
		}
	}

	public function setAttachments(KCommandContext $context)
	{
		$data			= $context['result'];
		$me				= KFactory::get('admin::com.ninjaboard.model.people')->getMe();

		if(is_a($data, 'KDatabaseRowsetInterface')) $data = (object) end($data->getData());
		$err			= null;
		$errors			= array();
		$identifier		= $this->getIdentifier();
		$destination	= JPATH_ROOT.'/media/'.$identifier->type.'_'.$identifier->package.'/attachments/';
		$attachments	= array();
		
		require_once JPATH_ROOT.'/components/com_media/helpers/media.php';

		$files = KRequest::get('files.attachments.name', 'raw', array());
		foreach ($files as $i => $file)
		{
			foreach (KRequest::get('files.attachments', 'raw') as $key => $values)
			{
				$attachment[$key] = KRequest::get('files.attachments.'.$key.'.'.$i, 'raw');
			}
			if(MediaHelper::canUpload($attachment, $err)) $attachments[] = $attachment;
			else $errors[] = array_merge($attachment, array('error' => $err));
		}
		
		foreach ($attachments as $attachment)
		{
			$upload = JFile::makeSafe(uniqid(time())).'.'.JFile::getExt($attachment['name']);

			JFile::upload($attachment['tmp_name'], $destination.$upload);
			KFactory::tmp('site::com.ninjaboard.model.attachments')
				->post($data->id)
				->getItem()
				->setData(array(
					'post' => $data->id, 
					'file' => $upload, 
					'name' => $attachment['name'],
					'joomla_user_id' => $me->id
				))
				->save();
		}
		
		foreach ($errors as $error)
		{
			JError::raiseWarning(21, sprintf(JText::_("%s couldn't upload because %s"), $error['name'], lcfirst($error['error'])));
		}
		
		foreach (KRequest::get('post.attachments', 'int', array()) as $attachment)
		{
			$item = KFactory::tmp('site::com.ninjaboard.model.attachments')
					->id($attachment)
					->getItem();

			if(JFile::exists($destination.$item->file)) JFile::delete($destination.$item->file);
			$item->delete();
		}		
	}

	protected function _actionSave($args)
	{
		$result = parent::_actionSave($args);

		if(is_a($result, 'KDatabaseRowAbstract'))
		{
			$row = $result;
		}
		elseif(is_a($result, 'KDatabaseRowsetAbstract'))
		{
			$row = $result[0];
		}
		else
		{
			return $result;
		}

		if($row->ninjaboard_topic_id && $row->id)
		{
			$this->setRedirect(JRoute::_('&view=topic&id='.$row->ninjaboard_topic_id.'&post='.$row->id, false).'#p'.$row->id);
		}

		return $result;
	}
	
	/*
	 * Generic cancel action
	 *
	 * @return 	void
	 */
	protected function _actionCancel()
	{
		$post	= KFactory::get($this->getModel())->getItem();

		$isset	= array(
			'post'	=> $post->id,
			'topic' => isset($this->_request->topic) && $this->_request->topic,
			'forum'	=> isset($this->_request->forum) && $this->_request->forum
		);

		if($isset['post']) {
			$this->_redirect = 'view=topic&id='.$post->ninjaboard_topic_id.'&post='.$post->id.'#p'.$post->id;
			
		} elseif($isset['topic']) {
			$this->_redirect = 'view=topic&id='.$this->_request->topic;
		} elseif($isset['forum']) {
			$this->_redirect = 'view=forum&id='.$this->_request->forum;
		} else {
			parent::_actionCancel();
		}
	}

	/**
	 * Checks wether the we can delete this post or not
	 *
	 * @TODO	currently if one post fails the permission check, 
	 *			the command chain is stopped by returning false.
	 *			Change it so posts that can be deleted, are deleted.
	 *
	 * @return boolean	returns false if permission check fails
	 */
	public function canDelete()
	{
		$user = KFactory::get('admin::com.ninjaboard.model.people')->getMe();
		$rows = KFactory::get($this->getModel())->getList();
		foreach($rows as $row)
		{
			$topic = KFactory::tmp('site::com.ninjaboard.model.topics')
																		->id($row->ninjaboard_topic_id)
																		->getItem();
			$forum = KFactory::tmp('site::com.ninjaboard.model.forums')
																		->id($topic->forum_id)
																		->getItem();

			// @TODO we migth want to add an option later, wether or not to allow users to delete their own post.
			if($forum->post_permissions < 3 && $row->created_by != $user->id) {
				JError::raiseError(403, JText::_("You don't have the permissions to delete others topics."));
				return false;
			}
		}
	}

	/**
	 * Updating forums, topics and users when a post successfully deleted.
	 *
	 * @param $param
	 */
	public function cleanupDelete(KCommandContext $context)
	{
		$rows = $context->result;
		$table = KFactory::get('site::com.ninjaboard.table.posts');
		
		foreach($rows as $row)
		{
			$topic	= KFactory::tmp('site::com.ninjaboard.model.topics')->id($row->ninjaboard_topic_id)->getItem();
			$query = KFactory::tmp('lib.koowa.database.query')->where('ninjaboard_topic_id', '=', $topic->id);
			$posts = $table->count($query);
		
			if($posts)
			{
				//Replies does not count the first post, thus we subtract by 1
				$topic->replies = $posts - 1;
				
				$query = KFactory::tmp('lib.koowa.database.query')
																->select('ninjaboard_post_id')
																->where('ninjaboard_topic_id', '=', $topic->id)
																->order('created_time', 'desc');
				$topic->last_post_id = $table->select($query, KDatabase::FETCH_FIELD);

				$topic->save();
			}
			
			if($topic->first_post_id == $row->id)
			{
				//If this is the last post, and only post in the topic, delete the topic.
				//If not, then don't delete it
				if(!$posts) $topic->delete();
				else return false;
			}
			
			//Update the forums' topics and posts count, and correct the last_post_id column
			$forums	= KFactory::tmp('site::com.ninjaboard.model.forums')->id($topic->forum_id)->getListWithParents();
			$forums->recount();
			
			if($row->created_by)
			{
				$user	= KFactory::tmp('site::com.ninjaboard.model.people')->id($row->created_by)->getItem();
				if(!$user->guest)
				{
					$query = KFactory::tmp('lib.koowa.database.query')->where('created_user_id', '=', $user->id);
					$user->posts = $table->count($query);

					$user->save();
				}
			}
		}
	}
}