CREATE TABLE IF NOT EXISTS `#__ninjaboard_assets` (
  `ninjaboard_asset_id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
  `level` tinyint(1) unsigned NOT NULL COMMENT 'The permission level. 0 = No Access, 1 = Has Access, 2 = and Can Create, 3 = and Can Manage',
  `name` varchar(255) NOT NULL COMMENT 'The unique name for the permission object',
  PRIMARY KEY (`ninjaboard_asset_id`),
  UNIQUE KEY `idx_asset_name` (`name`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_attachments` (
  `ninjaboard_attachment_id` SERIAL,
  `file` varchar(255) NOT NULL,
  `joomla_user_id` int(11) unsigned NOT NULL DEFAULT '0',
  `post_id` int(11) unsigned NOT NULL DEFAULT '0',
  `params` text NOT NULL,
  KEY `attach_post` (`post_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_profile_fields` (
  `ninjaboard_profile_field_id` SERIAL,
  `name` varchar(255) NOT NULL DEFAULT '',
  `description` text NOT NULL,
  `element` varchar(255) NOT NULL,
  `profile_field_type_id` int(11) unsigned NOT NULL DEFAULT '0',
  `size` int(7) unsigned NOT NULL DEFAULT '0',
  `min` int(5) NOT NULL,
  `max` int(5) NOT NULL,
  `enabled` tinyint(1) NOT NULL DEFAULT '1',
  `required` tinyint(1) NOT NULL DEFAULT '0',
  `disabled` tinyint(1) NOT NULL DEFAULT '0',
  `hidden` tinyint(1) NOT NULL DEFAULT '0',
  `registration` tinyint(1) NOT NULL DEFAULT '1',
  `params` text NOT NULL,
  `ordering` int(11) NOT NULL DEFAULT '0'
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_profile_values` (
  `ninjaboard_profile_value_id` SERIAL,
  `ninjaboard_profile_field_id` bigint(20) unsigned NOT NULL DEFAULT '0',
  `joomla_user_id` int(11) NOT NULL,
  `value` text NOT NULL,
  KEY `user_id` (`joomla_user_id`),
  KEY `field_id` (`ninjaboard_profile_field_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_forums` (
  `ninjaboard_forum_id` SERIAL,
  `parent_id` bigint(20) unsigned NOT NULL default '0'  COMMENT 'Needed for some converters',
  `level` int(10) unsigned NOT NULL DEFAULT '0'  COMMENT 'Hierarchy depth',
  `path` varchar(255) NOT NULL DEFAULT '/'  COMMENT 'Needed for the enumerated path implementation for the forum hierarchies',
  `enabled` tinyint(1) SIGNED  NOT NULL default 1,
  `title` varchar(255) NOT NULL DEFAULT '',
  `alias` varchar(255) NOT NULL,
  `description` text NOT NULL,
  `locked` tinyint(1) NOT NULL DEFAULT '0',
  `ordering` int(11) NOT NULL DEFAULT '1',
  `path_sort` text NOT NULL COMMENT 'Cached ordering by path (path+ninjaboard_forum_id)',
  `path_sort_title` text NOT NULL COMMENT 'path_sort with ids replaced with forum titles',
  `path_sort_ordering` text NOT NULL COMMENT 'path_sort with ids replaced with ordering values',
  `posts` int(11) NOT NULL DEFAULT '0',
  `topics` int(11) NOT NULL DEFAULT '0',
  `forum_type_id` int(11) unsigned NOT NULL DEFAULT '0',
  `checked_out` int(11) unsigned NOT NULL DEFAULT '0',
  `checked_out_time` datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
  `last_post_id` int(11) NOT NULL DEFAULT '0',
  `params` text NOT NULL,
  KEY `idx_alias` (`alias`),
  KEY `last_post_id` (`last_post_id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_posts` (
  `ninjaboard_post_id` SERIAL,
  `subject` varchar(255) NOT NULL DEFAULT '',
  `text` text NOT NULL,
  `created_time` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00'  COMMENT '',
  `created_user_id` int(11) unsigned NOT NULL default 0,
  `modified` datetime NOT NULL default '0000-00-00 00:00:00',
  `modified_user_id` int(11) unsigned NOT NULL default 0,
  `edit_reason` varchar(255) NOT NULL DEFAULT '',
  `user_ip` varchar(15) NOT NULL DEFAULT '',
  `locked` tinyint(1) NOT NULL DEFAULT '1',
  `access` tinyint(3) unsigned NOT NULL DEFAULT '0',
  `enabled` tinyint(1) NOT NULL DEFAULT '1',
  `params` text NOT NULL,
  `ninjaboard_topic_id` int(11) NOT NULL DEFAULT '0',
  `guest_name` varchar(255) DEFAULT '',
  `guest_email` varchar(255) DEFAULT '',
  KEY `user_id` (`created_user_id`),
  KEY `ninjaboard_topic_id` (`ninjaboard_topic_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_ranks` (
  `ninjaboard_rank_id` SERIAL,
  `title` varchar(255) NOT NULL DEFAULT '',
  `description` text NOT NULL,
  `min` int(11) NOT NULL DEFAULT '0',
  `enabled` tinyint(1) NOT NULL DEFAULT '1',
  `rank_file` text NOT NULL,
  `rank_thumb` text NOT NULL,
  `rank_type_id` int(11) unsigned NOT NULL DEFAULT '0',
  `params` text NOT NULL,
  `checked_out` int(11) unsigned NOT NULL DEFAULT '0',
  `checked_out_time` datetime NOT NULL DEFAULT '0000-00-00 00:00:00'
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_settings` (
  `ninjaboard_setting_id` SERIAL,
  `title` varchar(255) NOT NULL DEFAULT '',
  `default` tinyint(1) NOT NULL DEFAULT '0',
  `enabled` tinyint(1) NOT NULL DEFAULT '1',
  `ordering` int(11) NOT NULL DEFAULT '0',
  `params` text NOT NULL,
  `checked_out` int(11) unsigned NOT NULL DEFAULT '0',
  `checked_out_time` datetime NOT NULL DEFAULT '0000-00-00 00:00:00'
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_subscriptions` (
  `ninjaboard_subscription_id` SERIAL,
  `subs_type_id` int(11) unsigned NOT NULL DEFAULT '0',
  `joomla_user_id` int(11) unsigned NOT NULL DEFAULT '0',
  `guest_name` varchar(255) DEFAULT '',
  `guest_email` varchar(255) DEFAULT '',
  `id_subs_item` int(11) unsigned NOT NULL DEFAULT '0',
  `one_mail_or_many` tinyint(1) unsigned NOT NULL DEFAULT '0',
  `mail_sent` tinyint(1) unsigned NOT NULL DEFAULT '0',
  KEY `subs_user` (`subs_type_id`,`id_subs_item`,`joomla_user_id`),
  KEY `subs_item` (`subs_type_id`,`id_subs_item`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_topics` (
  `ninjaboard_topic_id` SERIAL,
  `replies` int(11) unsigned NOT NULL DEFAULT '0',
  `status` tinyint(3) NOT NULL DEFAULT '0',
  `vote` tinyint(1) NOT NULL DEFAULT '0',
  `topic_type_id` bigint(20) unsigned NOT NULL DEFAULT '0',
  `moved_from_forum_id` int(11) unsigned NOT NULL DEFAULT '0',
  `forum_id` int(11) NOT NULL DEFAULT '0',
  `first_post_id` int(11) unsigned NOT NULL DEFAULT '0',
  `last_post_id` int(11) unsigned NOT NULL DEFAULT '0',
  `hits` int(11) unsigned NOT NULL DEFAULT '0',
  `enabled` tinyint(1) NOT NULL DEFAULT '1',
  `sticky` tinyint(1) NOT NULL DEFAULT '1',
  `locked` tinyint(1) NOT NULL DEFAULT '1',
  `resolved` tinyint(1) NOT NULL DEFAULT '1',
  `access` tinyint(3) unsigned NOT NULL DEFAULT '0',
  `params` text NOT NULL,
  KEY `topic_type_id` (`topic_type_id`),
  KEY `status` (`status`),
  KEY `forum_id` (`forum_id`),
  KEY `moved_from_forum_id` (`moved_from_forum_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_people` (
  `ninjaboard_person_id` int(11) unsigned NOT NULL DEFAULT '0',
  `posts` mediumint(8) unsigned NOT NULL DEFAULT '0',
  `thumb` text NOT NULL,
  `avatar` text NOT NULL,
  `signature` text,
  `params` text NOT NULL,
  PRIMARY KEY (`ninjaboard_person_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_user_groups` (
  `ninjaboard_user_group_id` SERIAL,
  `parent_id` int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Adjacency List Reference Id',
  `path` varchar(255) NOT NULL DEFAULT '',
  `title` varchar(100) NOT NULL DEFAULT ''
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_user_group_maps` (
  `joomla_user_id` int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Foreign Key to #__users.id',
  `ninjaboard_user_group_id` int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Foreign Key to #__ninjaboard_user_groups.id',
  PRIMARY KEY (`joomla_user_id`,`ninjaboard_user_group_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_joomla_user_group_maps` (
  `joomla_gid` int(10) unsigned NOT NULL DEFAULT '0',
  `ninjaboard_gid` int(10) unsigned NOT NULL DEFAULT '0',
  PRIMARY KEY (`joomla_gid`,`ninjaboard_gid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_templates` (
  `ninjaboard_template_id` SERIAL,
  `template` varchar(255) NOT NULL,
  `description` varchar(255) NOT NULL,
  `params` text NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `#__ninjaboard_iconsets` (
  `ninjaboard_iconset_id` SERIAL,
  `iconset` varchar(255) NOT NULL,
  `description` varchar(255) NOT NULL,
  `params` text NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=utf8;