<?php
/**
 * @version		$Id: package.php 414 2010-08-06 09:48:31Z stian $
 * @package		Koowa
 * @copyright	Copyright (C) 2009 - 2010 Nooku. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link		http://www.nooku.org
 */

/**
 * Koowa Dashboard Controller
 *
 * Have an update action which lets you update the framework remotely.
 *
 * @author		Stian Didriksen <stian@ninjaforge.com>
 * @category	Koowa
 * @package     Koowa_Components
 */
class ComKoowaControllerPackage extends ComDefaultControllerDefault
{

	/**
	 * Constructor
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config)
	{
		parent::__construct($config);

		// Register apis
		Jloader::register('JInstaller', JPATH_LIBRARIES.'/joomla/installer/installer.php');
		Jloader::register('JArchive', JPATH_LIBRARIES.'/joomla/filesystem/archive.php');
		Jloader::register('JFile', JPATH_LIBRARIES.'/joomla/filesystem/file.php');
	}

	/**
	 * Display the view
	 *
	 * @return void
	 */
	public function displayView()
	{
		if(!JFolder::exists(JPATH_COMPONENT_ADMINISTRATOR.'/packages/')) return;
	
		$model	= KFactory::get($this->getModel());
		$data	= array();
		
		if(KRequest::has('get.folder', 'cmd'))
		{
			$package	= JPATH_COMPONENT_ADMINISTRATOR.'/packages/'.$model->file(false)->getItem() . '/';
			$installer	= JInstaller::getInstance();
			$installer->install($package);
			$data['html'] = $installer->get('extension.message');
			JFolder::delete($package);
		}
		elseif(KRequest::has('get.file', 'filename'))
		{
			$package			= JPATH_COMPONENT_ADMINISTRATOR.'/packages/'.$model->folder(false)->getItem();
			$data['package']	= JFile::stripExt(basename($package));
			JArchive::extract($package, JPATH_COMPONENT_ADMINISTRATOR.'/packages/'.$data['package']);
			JFile::delete($package);
			
			$xml = end(JFolder::files(JPATH_COMPONENT_ADMINISTRATOR.'/packages/'.$data['package'], '.xml$', 1, true));
			$xml = simplexml_load_file($xml);

			$attribs = $xml->attributes();

			$data['package_name'] = (string) $xml->name . ': ' . ucfirst($attribs['type']);
			$data['package_version'] = (string) '<strong>' . JText::_('OK') . '</strong> - ' .$xml->version . @$xml->version['status'];

			ob_start();
			echo KFactory::get($this->getView())
				->set('i', KRequest::get('get.i', 'int', 0))
				->set('package', $xml->name . ' ' . $xml->version . @$xml->version['status'])
				->setLayout('installing')
				->display();
			$data['html'] = ob_get_clean();
		}
		else
		{
			ob_start();
			echo KFactory::get($this->getView())
				->setLayout('default')
				->display();
			$data['html'] = ob_get_clean();

			$packages = $model->getList();

			sort($packages);

			foreach ($packages as $i => $package)
			{
				$data['packages'][] = $package;
				ob_start();
				echo KFactory::get($this->getView())
					->set('i', ++$i)
					->set('package', $package)
					->setLayout('unpacking')
					->display();
				$data['layouts'][] = ob_get_clean();
			}
			
			if($model->getTotal() < 1) $data['html'] = null;
		}

		echo json_encode($data);
	}
}