<?php
/**
 * @version		$Id: install.php 414 2010-08-06 09:48:31Z stian $
 * @package		Koowa
 * @copyright   Copyright (C) 2009 - 2010 Nooku. All rights reserved.
 * @license     GNU GPL <http://www.gnu.org/licenses/gpl.html>
 * @link        http://www.nooku.org
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// In order to allow multiple components being installed in go, this check is crucial
if(!function_exists('com_install'))
{
	/**
	 * This function is required by JInstallerComponent in order to run this script
	 *
	 * @return	boolean	true
	 */
	function com_install()
	{
		// If MySQLi isn't available at all, roll back
		return function_exists('mysqli_connect');
	}
}

if (!function_exists('mysqli_connect'))
{
	JError::raiseWarning(0, JText::_("We're sorry but your server isn't configured with the MySQLi database driver. Please contact your host an ask them to enable MySQLi for your server."));
	return false;
}

// Check if mysqli is active, if not, then enable it
if(JFactory::getApplication()->getCfg('dbtype') != 'mysqli')
{
	$path = JPATH_CONFIGURATION.DS.'configuration.php';
	if(JFile::exists($path))
	{
		JPath::setPermissions($path, '0777');
		$search		= JFile::read($path);
		$replaced	= str_replace('var $dbtype = \'mysql\';', 'var $dbtype = \'mysqli\';', $search);
		JFile::write($path, $replaced);
		JPath::setPermissions($path, '0644');
	}
	JError::raiseNotice(0, JText::_("Database configuration setting changed to 'mysqli'."));
}

// Define variables
$database	= JFactory::getDBO();
$type		= 'com';
$manifest	= simplexml_load_file($this->parent->getPath('manifest'));
$package	= (string) $manifest->name;
$source		= $this->parent->getPath('source'); // The install package dir
$version	= (string) $manifest->version;
$logo		= 'http://static.nooku.org/code/images/nooku-framework-80.png';

// Prevent the plugin row to be inserted more than once
$query = "SELECT COUNT(*) FROM `#__plugins` WHERE element = 'koowa' AND folder = 'system'";
$database->setQuery($query);
if(!$database->loadResult())
{
	// Insert in database
	$plugin = JTable::getInstance('plugin');
	$plugin->name = 'System - Koowa';
	$plugin->folder = 'system';
	$plugin->element = 'koowa';
	$plugin->published = 1;
	if (!$plugin->store()) {
		// Install failed, roll back changes
		$this->parent->abort(JText::_('Plugin').' '.JText::_('Install').': '.$database->stderr(true));
		return false;
	}
}

// Disable com_koowa from the admin menu until its realy for primetime
$query = "UPDATE `#__components` SET `enabled` = '0' WHERE `option` = 'com_koowa'";
$database->setQuery($query);
$database->query();


// Copy over the folders for the fw, com_default, mod_default
foreach ($manifest->framework->folder as $folder)
{
	$from	= isset($folder['src']) ? $folder['src'] : $folder;
	JFolder::copy($source.$from, JPATH_ROOT.$folder, false, true);
}
// Copy over the files for the plugins, creating the koowa folder if needed
if(!JFolder::exists(JPATH_ROOT.'/plugins/koowa')) JFolder::create(JPATH_ROOT.'/plugins/koowa');
foreach ($manifest->framework->file as $file)
{
	JFile::copy($source.$file, JPATH_ROOT.$file);
}

$versiontext = '<em>'.JText::_('You need at least %s to run Nooku Framework. You have: %s').'</em>';

// Preload the logo
$document = JFactory::getDocument();
$document->addScriptDeclaration('(new Image).src = "'.$logo.'";');

// If we have additional packages, move them to a safe place (or JInstaller will delete them)
// and later install them by using KInstaller
if(JFolder::exists($source.'/packages'))
{
	$packages = JPATH_ADMINISTRATOR.'/components/com_'.$package.'/packages';
	if(JFolder::exists($packages)) JFolder::delete($packages);
	JFolder::copy($source.'/packages', JPATH_ADMINISTRATOR.'/components/com_'.$package.'/packages');

	$document->addScript(JURI::root(1).'/media/com_koowa/js/install.js');
}

$document->addStyleSheet(JURI::root(1).'/media/com_koowa/css/install.css');

$config = JFactory::getConfig();
$class = $config->getValue('debug', null) ? 'debug' : null; ?>

<div id="install" class="<?php echo $class ?>"><h2 class="working"><?php echo JText::_('Please wait, checking for additional files to install'); ?></h2></div>
<table>
	<tbody valign="top">
		<tr>
			<td>
				
				<img src="<?php echo $logo ?>" alt="<?php echo JText::_('Nooku Framework Logo') ?>" width="190" height="80" />
			</td>
			<td width="100%">
				<table class="adminlist">
					<thead>
						<tr>
							<th><?php echo JText::_('Task') ?></th>
							<th width="30%"><?php echo JText::_('Status') ?></th>
						</tr>
					</thead>
					<tbody id="tasks">
						<tr class="row0">
							<td class="key hasTip" title="<?php echo JText::_('PHP Version') ?>::<?php echo sprintf($versiontext, 'PHP v5.2', phpversion()) ?>"><?php echo JText::_('PHP Version') ?></td>
							<td>
								<?php echo version_compare(phpversion(), '5.2', '>=')
									? '<strong>'.JText::_('OK').'</strong> - '.phpversion()
									: sprintf($versiontext, 'PHP v5.2', phpversion()) ?>
							</td>
						</tr>
						<tr class="row1">
							<td class="key hasTip" title="<?php echo JText::_('MySQL Version') ?>::<?php echo sprintf($versiontext, 'MySQL v5.0.41', $database->getVersion()) ?>"><?php echo JText::_('MySQL Version') ?></td>
							<td>
								<?php echo version_compare($database->getVersion(), '5.0.41', '>=')
								? '<strong>'.JText::_('OK').'</strong> - '.$database->getVersion()
								: sprintf($versiontext, 'MySQL v5.0.41', $database->getVersion()) ?>
							</td>
						</tr>
						<tr class="row0">
							<td class="key"><?php echo JText::_('Nooku Framework') ?></td>
							<td><strong><?php echo JText::_('Installed') ?></strong> - <?php echo $version ?></td>
						</tr>
					</tbody>
				</table>
			</td>
		</tr>
	</tbody>
</table>