<?php
/**
* @version		$Id: slug.php 414 2010-08-06 09:48:31Z stian $
* @category		Koowa
* @package      Koowa_Filter
* @copyright    Copyright (C) 2007 - 2010 Johan Janssens and Mathias Verraes. All rights reserved.
* @license      GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
* @link 		http://www.koowa.org
*/

/**
 * Slug filter
 *
 * @author		Johan Janssens <johan@koowa.org>
 * @category	Koowa
 * @package     Koowa_Filter
 */
class KFilterSlug extends KFilterAbstract
{
	/**
	 * Separator character / string to use for replacing non alphabetic characters 
	 * in generated slug
	 *
	 * @var	string
	 */
	protected $_separator;
	
	/**
	 * Maximum length the generated slug can have. If this is null the length of
	 * the slug column will be used.
	 *
	 * @var	integer
	 */
	protected $_length;
	
	/**
	 * Constructor
	 *
	 * @param 	object	An optional KConfig object with configuration options
	 */
	public function __construct(KConfig $config) 
	{
		parent::__construct($config);

		$this->_length    = $config->length;
		$this->_separator = $config->separator;
	}
	
	/**
     * Initializes the options for the object
     * 
     * Called from {@link __construct()} as a first step of object instantiation.
     *
     * @param 	object 	An optional KConfig object with configuration options
     * @return void
     */
	protected function _initialize(KConfig $config)
    {
    	$config->append(array(
    		'separator' => '-',
    		'length' 	=> 100
	  	));
    	
    	parent::_initialize($config);
   	}
	
	/**
	 * Validate a value
	 * 
	 * Returns true if the string only contains US-ASCII and does not contain
	 * any spaces
	 *
	 * @param	mixed	Variable to be validated
	 * @return	bool	True when the variable is valid
	 */
	protected function _validate($value)
	{
		return KFactory::tmp('lib.koowa.filter.cmd')->validate($value);
	}
	
	/**
	 * Sanitize a value
	 * 
	 * Replace all accented UTF-8 characters by unaccented ASCII-7 "equivalents", 
	 * replace whitespaces by hyphens and lowercase the result.
	 * 
	 * @param	scalar	Variable to be sanitized
	 * @return	scalar
	 */
	protected function _sanitize($value)
	{
		//remove any '-' from the string they will be used as concatonater
		$value = str_replace($this->_separator, ' ', $value);
		
		//convert to ascii characters
		$value = KFactory::tmp('lib.koowa.filter.ascii')->sanitize($value);
		
		//lowercase and trim
		$value = trim(strtolower($value));
		
		//remove any duplicate whitespace, and ensure all characters are alphanumeric
		$value = preg_replace(array('/\s+/','/[^A-Za-z0-9\-]/'), array($this->_separator,''), $value);
		
		//limit length
		if (strlen($value) > $this->_length) {
			$value = substr($value, 0, $this->_length);
		}
		
		return $value;
	}
}