<?php
/**
 * @version 	$Id: abstract.php 414 2010-08-06 09:48:31Z stian $
 * @category	Koowa
 * @package		Koowa_Loader
 * @subpackage 	Adapter
 * @copyright	Copyright (C) 2007 - 2010 Johan Janssens and Mathias Verraes. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 */

/**
 * Abstract Loader Adapter
 *
 * @author		Johan Janssens <johan@koowa.org>
 * @category	Koowa
 * @package     Koowa_Loader
 * @subpackage 	Adapter
 * @uses 		KIdentifier
 */
abstract class KLoaderAdapterAbstract implements KLoaderAdapterInterface
{
	/**
	 * Get the path based on a class name or an identifier
	 *
	 * @param  string|object  	The class name or an identifier -[application::]type.package.[.path].name
	 * @return string|false		Returns the path on success FALSE on failure
	 */
	public function path($identifier)
	{
		$path = false;
		
		if($identifier instanceof KIdentifierInterface) {
			$path = $this->_pathFromIdentifier($identifier);
		} else {
			$path = $this->_pathFromClassname($identifier);
		}
		
		return $path;
	}
	
	/**
	 * Get the path based on an identifier
	 *
	 * @param  object  			An Identifier object - [application::]type.package.[.path].name
	 * @return string|false		Returns the path on success FALSE on failure
	 */
	abstract protected function _pathFromIdentifier($identifier);
	
	/**
	 * Get the path based on a class name
	 *
	 * @param  string		  	The class name 
	 * @return string|false		Returns the path on success FALSE on failure
	 */
	abstract protected function _pathFromClassname($classname);	
}