<?php
/**
 * @version 	$Id: abstract.php 414 2010-08-06 09:48:31Z stian $
 * @category	Koowa
 * @package		Koowa_Database
 * @subpackage 	Behavior
 * @copyright	Copyright (C) 2007 - 2010 Johan Janssens and Mathias Verraes. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 */

/**
 * Abstract Template Filter
 *
 * @author		Johan Janssens <johan@koowa.org>
 * @category	Koowa
 * @package     Koowa_Template
 * @subpackage 	Filter
 */
abstract class KTemplateFilterAbstract extends KObject implements KTemplateFilterInterface
{
	/**
	 * The behavior priority
	 *
	 * @var KIdentifierInterface
	 */
	protected $_priority;
	
	/**
	 * Template object
	 *
	 * @var	object
	 */
    protected $_template;
	
	/**
	 * Constructor.
	 *
	 * @param 	object 	An optional KConfig object with configuration options
	 */
	public function __construct( KConfig $config = null) 
	{ 
		parent::__construct($config);
		
		$this->_priority = $config->priority;
	}
	
	/**
     * Initializes the options for the object
     *
     * Called from {@link __construct()} as a first step of object instantiation.
     *
     * @param 	object 	An optional KConfig object with configuration options
     * @return void
     */
	protected function _initialize(KConfig $config)
    {
    	$config->append(array(
			'priority'   => KCommandChain::PRIORITY_NORMAL,
	  	));

    	parent::_initialize($config);
   	}
	
	/**
	 * Get the object identifier
	 * 
	 * @return	KIdentifier	
	 * @see 	KObjectIdentifiable
	 */
	public function getIdentifier()
	{
		return $this->_identifier;
	}
	
	/**
	 * Get the priority of a behavior
	 *
	 * @return	integer The command priority
	 */
  	public function getPriority()
  	{
  		return $this->_priority;
  	}
  		
	/**
	 * Command handler
	 * 
	 * @param 	string  	The command name
	 * @param 	object   	The command context
	 * @return 	boolean		Always returns TRUE
	 */
	final public function execute( $name, KCommandContext $context) 
	{
		//Set the template
		$this->_template = $context->caller;
		
		//Set the data
		$data = $context->data;

		if($name == KTemplateFilter::MODE_READ && $this instanceof KTemplateFilterRead) {
			$this->read($data);
		}

		if($name == KTemplateFilter::MODE_WRITE && $this instanceof KTemplateFilterWrite) {
			$this->write($data);
		} 
		
		//Get the data
		$context->data = $data;
		
		//Reset the template
		$this->_template = null;
		
		//@TODO : Allows filters to return false and halt the filter chain
		return true;
	}
	
	/**
 	 * Method to extract key/value pairs out of a string with xml style attributes
 	 *
 	 * @param	string	String containing xml style attributes
 	 * @return	array	Key/Value pairs for the attributes
 	 */
	protected function _parseAttributes( $string )
	{
	 	$result = array(); 
	 	
	 	if(!empty($string))
	 	{
			$attr	= array();

			preg_match_all( '/([\w:-]+)[\s]?=[\s]?"([^"]*)"/i', $string, $attr );

			if (is_array($attr))
			{
				$numPairs = count($attr[1]);
				for($i = 0; $i < $numPairs; $i++ ) {
					 $result[$attr[1][$i]] = $attr[2][$i];
				}	
			}
	 	}
	 		
		return $result;
	}
}