<?php
/**
 * @version		$Id: html.php 414 2010-08-06 09:48:31Z stian $
 * @category	Koowa
 * @package     Koowa_View
 * @copyright	Copyright (C) 2007 - 2010 Johan Janssens and Mathias Verraes. All rights reserved.
 * @license		GNU GPLv2 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 * @link     	http://www.koowa.org
 */

/**
 * View HTML Class
 *
 * @author		Johan Janssens <johan@koowa.org>
 * @category	Koowa
 * @package     Koowa_View
 */
class KViewHtml extends KViewTemplate
{
	/**
     * Initializes the config for the object
     *
     * Called from {@link __construct()} as a first step of object instantiation.
     *
     * @param 	object 	An optional KConfig object with configuration options
     * @return  void
     */
    protected function _initialize(KConfig $config)
    {
    	$config->append(array(
			'mimetype'	  => 'text/html',
       	));
    	
    	parent::_initialize($config);
    	
    	$config->template_filters = array_merge($config->template_filters, array('form'));
    }
	
	/**
	 * Return the views output
	 * 
	 * This function will auto assign the model data to the view if the auto_assign
	 * property is set to TRUE. 
 	 *
	 * @return string 	The output of the view
	 */
	public function display()
	{
		$model = KFactory::get($this->getModel());
			
		//Auto-assign the state to the view
		$this->assign('state', $model->getState());
		
		//Auto-assign the data from the model
		if($this->_auto_assign)
		{
			//Get the view name
			$name  = $this->getName();
		
			//Assign the data of the model to the view
			if(KInflector::isPlural($name))
			{
				$this->assign($name, 	$model->getList())
					 ->assign('total',	$model->getTotal());
			}
			else
			{
				$this->assign($name, $model->getItem());
			}
		}
		
		//Load the template
		$template = $this->loadTemplate();
		
		//Render the template
		$this->output .= $template;
	
		return parent::display();
	}
}